"""GitHub rate limit data class."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase


class GitHubRateLimitResourceModel(GitHubDataModelBase):
    """GitHub rate limit resource data class."""

    limit: int | None = None
    used: int | None = None
    remaining: int | None = None
    reset: int | None = None


class GitHubRateLimitResourcesModel(GitHubDataModelBase):
    """GitHub rate limit resources data class."""

    core: GitHubRateLimitResourceModel | None = None
    search: GitHubRateLimitResourceModel | None = None
    graphql: GitHubRateLimitResourceModel | None = None
    dependency_snapshots: GitHubRateLimitResourceModel | None = None
    integration_manifest: GitHubRateLimitResourceModel | None = None
    source_import: GitHubRateLimitResourceModel | None = None
    code_scanning_upload: GitHubRateLimitResourceModel | None = None
    actions_runner_registration: GitHubRateLimitResourceModel | None = None
    scim: GitHubRateLimitResourceModel | None = None

    def _generate_core(self, data: Any) -> GitHubRateLimitResourceModel:
        return GitHubRateLimitResourceModel(data) if data else None

    def _generate_search(self, data: Any) -> GitHubRateLimitResourceModel:
        return GitHubRateLimitResourceModel(data) if data else None

    def _generate_graphql(self, data: Any) -> GitHubRateLimitResourceModel:
        return GitHubRateLimitResourceModel(data) if data else None

    def _generate_integration_manifest(self, data: Any) -> GitHubRateLimitResourceModel:
        return GitHubRateLimitResourceModel(data) if data else None

    def _generate_source_import(self, data: Any) -> GitHubRateLimitResourceModel:
        return GitHubRateLimitResourceModel(data) if data else None

    def _generate_code_scanning_upload(self, data: Any) -> GitHubRateLimitResourceModel:
        return GitHubRateLimitResourceModel(data) if data else None

    def _generate_dependency_snapshots(self, data: Any) -> GitHubRateLimitResourceModel:
        return GitHubRateLimitResourceModel(data) if data else None


class GitHubRateLimitModel(GitHubDataModelBase):
    """GitHub rate limit data class."""

    rate: GitHubRateLimitResourceModel | None = None
    resources: GitHubRateLimitResourcesModel | None = None

    def _generate_rate(self, data: Dict[str, Any]) -> GitHubRateLimitResourceModel:
        """Generate rate limit resource model."""
        return GitHubRateLimitResourceModel(data) if data else None

    def _generate_resources(self, data: Dict[str, Any]) -> GitHubRateLimitResourcesModel:
        """Generate rate limit resources model."""
        return GitHubRateLimitResourcesModel(data) if data else None



"""Test client construction"""
# pylint: disable=missing-docstring,protected-access
from aiohttp import ClientSession
import pytest

from aiogithubapi.client import GitHubClient
from aiogithubapi.const import BASE_API_HEADERS, DEFAULT_USER_AGENT, HEADER_GITHUB_API_VERSION, GitHubClientKwarg

from tests.common import TOKEN


@pytest.mark.asyncio
async def test_client_construction_defaults(
    client_session: ClientSession, caplog: pytest.CaptureFixture
):
    client = GitHubClient(session=client_session)
    base_request_data = client._base_request_data

    assert client._session == client_session
    assert base_request_data.base_url == "https://api.github.com"
    assert base_request_data.timeout == 20
    assert base_request_data.token is None
    assert base_request_data.kwargs == {}
    assert base_request_data.headers == BASE_API_HEADERS
    assert "Authorization" not in base_request_data.headers
    assert base_request_data.headers["User-Agent"] == DEFAULT_USER_AGENT
    assert (
        "User-Agent not set. Set this with passing a user-agent header or the client_name argument."
        in caplog.text
    )


@pytest.mark.asyncio
async def test_client_construction_with_token(client_session: ClientSession):
    client = GitHubClient(session=client_session, token=TOKEN)
    base_request_data = client._base_request_data
    assert base_request_data.token == TOKEN
    assert base_request_data.headers == {
        **BASE_API_HEADERS,
        "Authorization": "token xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
    }


@pytest.mark.asyncio
async def test_client_construction_with_api_version(client_session: ClientSession):
    client = GitHubClient(session=client_session, token=TOKEN, api_version="3000-01-01")
    base_request_data = client._base_request_data
    assert base_request_data.token == TOKEN
    assert base_request_data.headers == {
        **BASE_API_HEADERS,
        HEADER_GITHUB_API_VERSION: "3000-01-01",
        "Authorization": "token xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
    }


@pytest.mark.asyncio
async def test_client_construction_with_kwargs_timeout(client_session: ClientSession):
    client = GitHubClient(session=client_session, **{GitHubClientKwarg.TIMEOUT: 10})
    base_request_data = client._base_request_data
    assert base_request_data.timeout == 10


@pytest.mark.asyncio
async def test_client_construction_with_kwargs_base_url(client_session: ClientSession):
    client = GitHubClient(
        session=client_session, **{GitHubClientKwarg.BASE_URL: "https://example.com"}
    )
    base_request_data = client._base_request_data
    assert base_request_data.base_url == "https://example.com"


@pytest.mark.asyncio
async def test_client_construction_with_kwargs_headers(client_session: ClientSession):
    client = GitHubClient(
        session=client_session, **{GitHubClientKwarg.HEADERS: {"User-Agent": "test/client"}}
    )
    base_request_data = client._base_request_data
    assert base_request_data.headers["User-Agent"] == "test/client"


@pytest.mark.asyncio
async def test_client_construction_with_kwargs_client_name(client_session: ClientSession):
    client = GitHubClient(
        session=client_session, **{GitHubClientKwarg.CLIENT_NAME: "test_client/1.2.3"}
    )
    base_request_data = client._base_request_data
    assert base_request_data.headers["User-Agent"] == "test_client/1.2.3"



"""
Generated by generate/generate.py - 2020-08-02 10:38:59.444717
"""
from aiogithubapi.objects.repos.fork import AIOGitHubAPIReposFork

from tests.legacy.responses.repos.fork_fixtrue import fork_fixtrue_response


def test_fork(fork_fixtrue_response):
    obj = AIOGitHubAPIReposFork(fork_fixtrue_response)
    assert obj.id == fork_fixtrue_response["id"]
    assert obj.node_id == fork_fixtrue_response["node_id"]
    assert obj.name == fork_fixtrue_response["name"]
    assert obj.full_name == fork_fixtrue_response["full_name"]
    assert obj.owner.login == fork_fixtrue_response["owner"]["login"]
    assert obj.owner.id == fork_fixtrue_response["owner"]["id"]
    assert obj.owner.node_id == fork_fixtrue_response["owner"]["node_id"]
    assert obj.owner.avatar_url == fork_fixtrue_response["owner"]["avatar_url"]
    assert obj.owner.gravatar_id == fork_fixtrue_response["owner"]["gravatar_id"]
    assert obj.owner.url == fork_fixtrue_response["owner"]["url"]
    assert obj.owner.html_url == fork_fixtrue_response["owner"]["html_url"]
    assert obj.owner.followers_url == fork_fixtrue_response["owner"]["followers_url"]
    assert obj.owner.following_url == fork_fixtrue_response["owner"]["following_url"]
    assert obj.owner.gists_url == fork_fixtrue_response["owner"]["gists_url"]
    assert obj.owner.starred_url == fork_fixtrue_response["owner"]["starred_url"]
    assert obj.owner.subscriptions_url == fork_fixtrue_response["owner"]["subscriptions_url"]
    assert obj.owner.organizations_url == fork_fixtrue_response["owner"]["organizations_url"]
    assert obj.owner.repos_url == fork_fixtrue_response["owner"]["repos_url"]
    assert obj.owner.events_url == fork_fixtrue_response["owner"]["events_url"]
    assert obj.owner.received_events_url == fork_fixtrue_response["owner"]["received_events_url"]
    assert obj.owner.type == fork_fixtrue_response["owner"]["type"]
    assert obj.owner.site_admin == fork_fixtrue_response["owner"]["site_admin"]
    assert obj.private == fork_fixtrue_response["private"]
    assert obj.html_url == fork_fixtrue_response["html_url"]
    assert obj.description == fork_fixtrue_response["description"]
    assert obj.fork == fork_fixtrue_response["fork"]
    assert obj.url == fork_fixtrue_response["url"]
    assert obj.archive_url == fork_fixtrue_response["archive_url"]
    assert obj.assignees_url == fork_fixtrue_response["assignees_url"]
    assert obj.blobs_url == fork_fixtrue_response["blobs_url"]
    assert obj.branches_url == fork_fixtrue_response["branches_url"]
    assert obj.collaborators_url == fork_fixtrue_response["collaborators_url"]
    assert obj.comments_url == fork_fixtrue_response["comments_url"]
    assert obj.commits_url == fork_fixtrue_response["commits_url"]
    assert obj.compare_url == fork_fixtrue_response["compare_url"]
    assert obj.contents_url == fork_fixtrue_response["contents_url"]
    assert obj.contributors_url == fork_fixtrue_response["contributors_url"]
    assert obj.deployments_url == fork_fixtrue_response["deployments_url"]
    assert obj.downloads_url == fork_fixtrue_response["downloads_url"]
    assert obj.events_url == fork_fixtrue_response["events_url"]
    assert obj.forks_url == fork_fixtrue_response["forks_url"]
    assert obj.git_commits_url == fork_fixtrue_response["git_commits_url"]
    assert obj.git_refs_url == fork_fixtrue_response["git_refs_url"]
    assert obj.git_tags_url == fork_fixtrue_response["git_tags_url"]
    assert obj.git_url == fork_fixtrue_response["git_url"]
    assert obj.issue_comment_url == fork_fixtrue_response["issue_comment_url"]
    assert obj.issue_events_url == fork_fixtrue_response["issue_events_url"]
    assert obj.issues_url == fork_fixtrue_response["issues_url"]
    assert obj.keys_url == fork_fixtrue_response["keys_url"]
    assert obj.labels_url == fork_fixtrue_response["labels_url"]
    assert obj.languages_url == fork_fixtrue_response["languages_url"]
    assert obj.merges_url == fork_fixtrue_response["merges_url"]
    assert obj.milestones_url == fork_fixtrue_response["milestones_url"]
    assert obj.notifications_url == fork_fixtrue_response["notifications_url"]
    assert obj.pulls_url == fork_fixtrue_response["pulls_url"]
    assert obj.releases_url == fork_fixtrue_response["releases_url"]
    assert obj.ssh_url == fork_fixtrue_response["ssh_url"]
    assert obj.stargazers_url == fork_fixtrue_response["stargazers_url"]
    assert obj.statuses_url == fork_fixtrue_response["statuses_url"]
    assert obj.subscribers_url == fork_fixtrue_response["subscribers_url"]
    assert obj.subscription_url == fork_fixtrue_response["subscription_url"]
    assert obj.tags_url == fork_fixtrue_response["tags_url"]
    assert obj.teams_url == fork_fixtrue_response["teams_url"]
    assert obj.trees_url == fork_fixtrue_response["trees_url"]
    assert obj.clone_url == fork_fixtrue_response["clone_url"]
    assert obj.mirror_url == fork_fixtrue_response["mirror_url"]
    assert obj.hooks_url == fork_fixtrue_response["hooks_url"]
    assert obj.svn_url == fork_fixtrue_response["svn_url"]
    assert obj.homepage == fork_fixtrue_response["homepage"]
    assert obj.language == fork_fixtrue_response["language"]
    assert obj.forks_count == fork_fixtrue_response["forks_count"]
    assert obj.stargazers_count == fork_fixtrue_response["stargazers_count"]
    assert obj.watchers_count == fork_fixtrue_response["watchers_count"]
    assert obj.size == fork_fixtrue_response["size"]
    assert obj.default_branch == fork_fixtrue_response["default_branch"]
    assert obj.open_issues_count == fork_fixtrue_response["open_issues_count"]
    assert obj.is_template == fork_fixtrue_response["is_template"]
    assert obj.topics[0] == fork_fixtrue_response["topics"][0]
    assert obj.has_issues == fork_fixtrue_response["has_issues"]
    assert obj.has_projects == fork_fixtrue_response["has_projects"]
    assert obj.has_wiki == fork_fixtrue_response["has_wiki"]
    assert obj.has_pages == fork_fixtrue_response["has_pages"]
    assert obj.has_downloads == fork_fixtrue_response["has_downloads"]
    assert obj.archived == fork_fixtrue_response["archived"]
    assert obj.disabled == fork_fixtrue_response["disabled"]
    assert obj.visibility == fork_fixtrue_response["visibility"]
    assert obj.pushed_at == fork_fixtrue_response["pushed_at"]
    assert obj.created_at == fork_fixtrue_response["created_at"]
    assert obj.updated_at == fork_fixtrue_response["updated_at"]
    assert obj.permissions.admin == fork_fixtrue_response["permissions"]["admin"]
    assert obj.permissions.push == fork_fixtrue_response["permissions"]["push"]
    assert obj.permissions.pull == fork_fixtrue_response["permissions"]["pull"]
    assert obj.template_repository == fork_fixtrue_response["template_repository"]
    assert obj.temp_clone_token == fork_fixtrue_response["temp_clone_token"]
    assert obj.delete_branch_on_merge == fork_fixtrue_response["delete_branch_on_merge"]
    assert obj.subscribers_count == fork_fixtrue_response["subscribers_count"]
    assert obj.network_count == fork_fixtrue_response["network_count"]
    assert obj.license.key == fork_fixtrue_response["license"]["key"]
    assert obj.license.name == fork_fixtrue_response["license"]["name"]
    assert obj.license.spdx_id == fork_fixtrue_response["license"]["spdx_id"]
    assert obj.license.url == fork_fixtrue_response["license"]["url"]
    assert obj.license.node_id == fork_fixtrue_response["license"]["node_id"]



"""Fixtures."""
# pylint: disable=missing-docstring
import pytest


@pytest.fixture()
def branch_response():
    return {
        "name": "main",
        "commit": {
            "sha": "7fd1a60b01f91b314f59955a4e4d4e80d8edf11d",
            "node_id": "MDY6Q29tbWl0N2ZkMWE2MGIwMWY5MWIzMTRmNTk5NTVhNGU0ZDRlODBkOGVkZjExZA==",
            "commit": {
                "author": {
                    "name": "The Octocat",
                    "date": "2012-03-06T15:06:50-08:00",
                    "email": "octocat@nowhere.com",
                },
                "url": "https://api.github.com/repos/octocat/Hello-World/git/commits/7fd1a60b01f91b314f59955a4e4d4e80d8edf11d",
                "message": "Merge pull request #6 from Spaceghost/patch-1\n\nNew line at end of file.",
                "tree": {
                    "sha": "b4eecafa9be2f2006ce1b709d6857b07069b4608",
                    "url": "https://api.github.com/repos/octocat/Hello-World/git/trees/b4eecafa9be2f2006ce1b709d6857b07069b4608",
                },
                "committer": {
                    "name": "The Octocat",
                    "date": "2012-03-06T15:06:50-08:00",
                    "email": "octocat@nowhere.com",
                },
                "verification": {
                    "verified": False,
                    "reason": "unsigned",
                    "signature": None,
                    "payload": None,
                },
            },
            "author": {
                "gravatar_id": "",
                "avatar_url": "https://secure.gravatar.com/avatar/7ad39074b0584bc555d0417ae3e7d974?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-140.png",
                "url": "https://api.github.com/users/octocat",
                "id": 583231,
                "login": "octocat",
            },
            "parents": [
                {
                    "sha": "553c2077f0edc3d5dc5d17262f6aa498e69d6f8e",
                    "url": "https://api.github.com/repos/octocat/Hello-World/commits/553c2077f0edc3d5dc5d17262f6aa498e69d6f8e",
                },
                {
                    "sha": "762941318ee16e59dabbacb1b4049eec22f0d303",
                    "url": "https://api.github.com/repos/octocat/Hello-World/commits/762941318ee16e59dabbacb1b4049eec22f0d303",
                },
            ],
            "url": "https://api.github.com/repos/octocat/Hello-World/commits/7fd1a60b01f91b314f59955a4e4d4e80d8edf11d",
            "committer": {
                "gravatar_id": "",
                "avatar_url": "https://secure.gravatar.com/avatar/7ad39074b0584bc555d0417ae3e7d974?d=https://a248.e.akamai.net/assets.github.com%2Fimages%2Fgravatars%2Fgravatar-140.png",
                "url": "https://api.github.com/users/octocat",
                "id": 583231,
                "login": "octocat",
            },
        },
        "_links": {
            "html": "https://github.com/octocat/Hello-World/tree/main",
            "self": "https://api.github.com/repos/octocat/Hello-World/branches/main",
        },
        "protected": True,
        "protection": {
            "enabled": True,
            "required_status_checks": {
                "enforcement_level": "non_admins",
                "contexts": ["ci-test", "linter"],
            },
        },
        "protection_url": "https://api.github.com/repos/octocat/hello-world/branches/main/protection",
    }



"""Test rate_limit."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI

from tests.common import MockedRequests


@pytest.mark.asyncio
async def test_rate_limit(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.rate_limit()
    assert response.status == 200
    assert response.data.resources.core.limit == 5000
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/rate_limit"



"""
Asynchronous Python client for the GitHub API https://github.com/ludeeus/aiogithubapi
"""

from .common.exceptions import (
    AIOGitHubAPIAuthenticationException,
    AIOGitHubAPIException,
    AIOGitHubAPINotModifiedException,
    AIOGitHubAPIRatelimitException,
)
from .const import (
    DeviceFlowError,
    GitHubClientKwarg,
    GitHubIssueLockReason,
    GitHubRequestKwarg,
    HttpStatusCode,
    Repository,
)
from .device import GitHubDeviceAPI
from .exceptions import (
    GitHubAuthenticationException,
    GitHubConnectionException,
    GitHubException,
    GitHubGraphQLException,
    GitHubNotFoundException,
    GitHubNotModifiedException,
    GitHubPayloadException,
    GitHubPermissionException,
    GitHubRatelimitException,
)
from .github import GitHub as GitHubAPI
from .legacy.device import AIOGitHubAPIDeviceLogin as GitHubDevice
from .legacy.github import AIOGitHubAPI as GitHub
from .models.base import GitHubBase
from .models.clones import GitHubClonesModel
from .models.commit import GitHubCommitModel
from .models.contents import GitHubContentsModel
from .models.device_login import GitHubLoginDeviceModel
from .models.events import GitHubEventModel
from .models.git_tree import GitHubGitTreeEntryModel, GitHubGitTreeModel
from .models.issue import GitHubIssueModel
from .models.issue_comment import GitHubIssueCommentModel
from .models.label import GitHubLabelModel
from .models.license import GitHubLicenseModel
from .models.login_oauth import GitHubLoginOauthModel
from .models.meta import GitHubMetaModel
from .models.milestone import GitHubMilestoneModel
from .models.notification import GitHubNotificationModel
from .models.organization import GitHubOrganizationMinimalModel, GitHubOrganizationModel
from .models.owner import GitHubOwnerModel
from .models.permissions import GitHubPermissionsModel
from .models.projects import GitHubProjectModel
from .models.pull_request import GitHubPullRequestModel
from .models.rate_limit import (
    GitHubRateLimitModel,
    GitHubRateLimitResourceModel,
    GitHubRateLimitResourcesModel,
)
from .models.reaction import GitHubReactionModel
from .models.release import GitHubReleaseAssetModel, GitHubReleaseModel
from .models.repository import GitHubRepositoryModel
from .models.request_data import GitHubBaseRequestDataModel
from .models.response import GitHubResponseHeadersModel, GitHubResponseModel
from .models.tag import GitHubTagModel
from .models.user import (
    GitHubAuthenticatedUserModel,
    GitHubBaseUserModel,
    GitHubUserModel,
    GitHubUserPlanModel,
)
from .models.views import GitHubViewsModel



"""Dataclass to hold base request details."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Dict

from aiohttp.hdrs import AUTHORIZATION, USER_AGENT

from ..const import (
    BASE_API_HEADERS,
    BASE_API_URL,
    DEFAULT_USER_AGENT,
    HEADER_GITHUB_API_VERSION,
    GitHubClientKwarg,
)
from .base import GitHubBase


@dataclass
class GitHubBaseRequestDataModel(GitHubBase):
    """Dataclass to hold base request details."""

    kwargs: Dict[GitHubClientKwarg, Any]
    token: str | None = None
    api_version: str | None = None

    def __post_init__(self):
        """Check user agent."""
        if self.headers[USER_AGENT] == DEFAULT_USER_AGENT:
            self.logger.debug(
                "User-Agent not set. Set this with passing a user-agent header or the client_name argument."
            )

    def request_url(self, endpoint: str) -> str:
        """Generate full request url."""
        return f"{self.base_url}{endpoint}"

    @property
    def timeout(self) -> int:
        """Return timeout."""
        return self.kwargs.get(GitHubClientKwarg.TIMEOUT) or 20

    @property
    def base_url(self) -> str:
        """Return the base url."""
        return self.kwargs.get(GitHubClientKwarg.BASE_URL) or BASE_API_URL

    @property
    def headers(self) -> Dict[str, str]:
        """Return base request headers."""
        headers = BASE_API_HEADERS.copy()
        if self.token:
            headers[AUTHORIZATION] = f"token {self.token}"
        if kwarg_headers := self.kwargs.get(GitHubClientKwarg.HEADERS):
            headers.update(kwarg_headers)
        if client_name := self.kwargs.get(GitHubClientKwarg.CLIENT_NAME):
            headers[USER_AGENT] = client_name
        if self.api_version is not None:
            headers[HEADER_GITHUB_API_VERSION] = self.api_version
        return headers



"""
Generated by generate/generate.py - 2020-09-08 18:45:20.581868
"""
from aiogithubapi.objects.login.oauth import AIOGitHubAPILoginOauth

from tests.legacy.responses.login.oauth_fixtrue import oauth_fixtrue_response


def test_oauth(oauth_fixtrue_response):
    obj = AIOGitHubAPILoginOauth(oauth_fixtrue_response)
    assert obj.access_token == oauth_fixtrue_response["access_token"]
    assert obj.token_type == oauth_fixtrue_response["token_type"]
    assert obj.scope == oauth_fixtrue_response["scope"]



"""Helpers for AIOGitHubAPI."""

import asyncio
from typing import Optional

import aiohttp
from aiohttp.client_exceptions import ClientError
import backoff

from ..common.const import HTTP_STATUS_CODE_GOOD_LIST, HttpMethod, HttpStatusCode
from ..common.exceptions import (
    AIOGitHubAPIAuthenticationException,
    AIOGitHubAPIException,
    AIOGitHubAPINotModifiedException,
    AIOGitHubAPIRatelimitException,
)
from ..objects.base import AIOGitHubAPIResponse


def short_sha(sha: str) -> str:
    """Return the first 7 characters of the sha."""
    return sha[0:7]


def short_message(message: str) -> str:
    """Return the first line of a message"""
    return message.split("\n")[0]


@backoff.on_exception(
    backoff.expo,
    (ClientError, asyncio.CancelledError, TimeoutError, KeyError),
    max_tries=5,
    logger=None,
)
async def async_call_api(
    session: aiohttp.ClientSession,
    method: HttpMethod,
    url: str,
    headers: dict,
    params: Optional[dict] = None,
    data: dict or str or None = None,
    jsondata: bool = True,
    returnjson: bool = True,
) -> AIOGitHubAPIResponse:
    """Execute the API call."""
    response = AIOGitHubAPIResponse()

    async with asyncio.timeout(20):
        if method == HttpMethod.GET:
            result = await session.get(url, params=params or {}, headers=headers)
        else:
            if jsondata:
                result = await session.post(
                    url,
                    params=params or {},
                    headers=headers,
                    json=data or {},
                )
            else:
                result = await session.post(
                    url,
                    params=params or {},
                    headers=headers,
                    data=data or "",
                )

        response.status = result.status
        response.headers = result.headers

        if response.status == HttpStatusCode.RATELIMIT:
            raise AIOGitHubAPIRatelimitException("GitHub Ratelimit error")

        if response.status == HttpStatusCode.UNAUTHORIZED:
            raise AIOGitHubAPIAuthenticationException(HttpStatusCode.UNAUTHORIZED)

        if response.status == HttpStatusCode.NOT_MODIFIED:
            raise AIOGitHubAPINotModifiedException(f"Response from {url} was not modified.")

        if response.status not in HTTP_STATUS_CODE_GOOD_LIST:
            raise AIOGitHubAPIException(
                f"GitHub returned {HttpStatusCode(response.status)} for {url}"
            )

        if returnjson:
            response.data = await result.json()
        else:
            response.data = await result.text()

        if isinstance(response.data, dict):
            if response.data.get("message"):
                if response.data["message"] == "Bad credentials":
                    raise AIOGitHubAPIAuthenticationException("Access token is not valid!")
                raise AIOGitHubAPIException(response.data)

    return response



"""
Methods for the releases namespace

https://docs.github.com/en/rest/reference/repos#releases
"""

from __future__ import annotations

from typing import Any, Dict, List

from ..const import GitHubRequestKwarg, RepositoryType
from ..models.release import GitHubReleaseModel
from ..models.response import GitHubResponseModel
from .base import BaseNamespace


class GitHubReleasesNamespace(BaseNamespace):
    """Methods for the releases namespace"""

    async def list(
        self,
        repository: RepositoryType,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[List[GitHubReleaseModel]]:
        """
        List releases

        This returns a list of releases, which does not include
        regular Git tags that have not been associated with a release.

         **Arguments**:

         `repository`

         The repository to return the tree for, example "octocat/hello-world"

        https://docs.github.com/en/rest/reference/repos#list-releases
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository}/releases",
            **kwargs,
        )
        response.data = [GitHubReleaseModel(data) for data in response.data]
        return response

    async def latest(
        self,
        repository: RepositoryType,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubReleaseModel]:
        """
        Get the latest release

        This returns the latest release for the specified repository.

        **Arguments**:

        `repository`

        The repository to return the latest release for, example "octocat/hello-world"

        https://docs.github.com/en/rest/reference/repos#get-the-latest-release
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository}/releases/latest",
            **kwargs,
        )
        response.data = GitHubReleaseModel(response.data)
        return response



"""
Class object for AIOGitHubAPIUsersUser
Documentation: https://docs.github.com/en/rest/reference/users#get-a-user

Generated by generate/generate.py - 2020-08-02 10:35:28.920747
"""

from ..base import AIOGitHubAPIBase


class AIOGitHubAPIUsersUser(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def company(self):
        return self.attributes.get("company", "")

    @property
    def blog(self):
        return self.attributes.get("blog", "")

    @property
    def location(self):
        return self.attributes.get("location", "")

    @property
    def email(self):
        return self.attributes.get("email", "")

    @property
    def hireable(self):
        return self.attributes.get("hireable", False)

    @property
    def bio(self):
        return self.attributes.get("bio", "")

    @property
    def twitter_username(self):
        return self.attributes.get("twitter_username", "")

    @property
    def public_repos(self):
        return self.attributes.get("public_repos", None)

    @property
    def public_gists(self):
        return self.attributes.get("public_gists", None)

    @property
    def followers(self):
        return self.attributes.get("followers", None)

    @property
    def following(self):
        return self.attributes.get("following", None)

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at", "")



from aiogithubapi.helpers import short_message, short_sha


def test_short_sha():
    assert short_sha("123456789") == "1234567"


def test_short_message():
    assert short_message("show\nno show") == "show"



"""Test repository dataclass."""
from io import BytesIO
import json
from unittest.mock import patch
from aiogithubapi import Repository
from aiogithubapi.helpers import repository_full_name


def test_repository():
    """Test repository dataclass."""
    assert (
        repository_full_name(Repository(owner="octocat", repo="Hello-World"))
        == "octocat/Hello-World"
    )
    assert repository_full_name("octocat/Hello-World") == "octocat/Hello-World"
    assert (
        repository_full_name({"owner": "octocat", "repo": "Hello-World"}) == "octocat/Hello-World"
    )



"""Custom exceptions for aiogithubapi."""


class GitHubException(Exception):
    """
    This is raised when unknown exceptions occur.

    And it's used as a base for all other exceptions
    so if you want to catch all GitHub related errors
    you should catch this base exception.
    """


class GitHubConnectionException(GitHubException):
    """This is raised when there is a connection issue with GitHub."""


class GitHubRatelimitException(GitHubException):
    """This is raised when the ratelimit is reached."""


class GitHubNotFoundException(GitHubException):
    """This is raised when the requested resource is not found."""


class GitHubPayloadException(GitHubException):
    """This is raised when the payload is invalid."""


class GitHubGraphQLException(GitHubException):
    """This is raised when the response from GraphQL calls have errors."""


class GitHubPermissionException(GitHubException):
    """This is raised when the user has no permission to do the requested resource."""


class GitHubNotModifiedException(GitHubException):
    """This is raised when the providede ETag matches and the content has not been modified."""


class GitHubAuthenticationException(GitHubException):
    """This is raised when we receive an authentication issue."""



"""
Generated by generate/generate.py - 2020-08-02 10:29:03.550853
"""
from aiogithubapi.objects.repos.branch import AIOGitHubAPIReposBranch

from tests.legacy.responses.repos.branch_fixtrue import branch_fixtrue_response


def test_branch(branch_fixtrue_response):
    obj = AIOGitHubAPIReposBranch(branch_fixtrue_response)
    assert obj.name == branch_fixtrue_response["name"]
    assert obj.commit.sha == branch_fixtrue_response["commit"]["sha"]
    assert obj.commit.node_id == branch_fixtrue_response["commit"]["node_id"]
    assert (
        obj.commit.commit.author.name
        == branch_fixtrue_response["commit"]["commit"]["author"]["name"]
    )
    assert (
        obj.commit.commit.author.date
        == branch_fixtrue_response["commit"]["commit"]["author"]["date"]
    )
    assert (
        obj.commit.commit.author.email
        == branch_fixtrue_response["commit"]["commit"]["author"]["email"]
    )
    assert obj.commit.commit.url == branch_fixtrue_response["commit"]["commit"]["url"]
    assert obj.commit.commit.message == branch_fixtrue_response["commit"]["commit"]["message"]
    assert obj.commit.commit.tree.sha == branch_fixtrue_response["commit"]["commit"]["tree"]["sha"]
    assert obj.commit.commit.tree.url == branch_fixtrue_response["commit"]["commit"]["tree"]["url"]
    assert (
        obj.commit.commit.committer.name
        == branch_fixtrue_response["commit"]["commit"]["committer"]["name"]
    )
    assert (
        obj.commit.commit.committer.date
        == branch_fixtrue_response["commit"]["commit"]["committer"]["date"]
    )
    assert (
        obj.commit.commit.committer.email
        == branch_fixtrue_response["commit"]["commit"]["committer"]["email"]
    )
    assert (
        obj.commit.commit.verification.verified
        == branch_fixtrue_response["commit"]["commit"]["verification"]["verified"]
    )
    assert (
        obj.commit.commit.verification.reason
        == branch_fixtrue_response["commit"]["commit"]["verification"]["reason"]
    )
    assert (
        obj.commit.commit.verification.signature
        == branch_fixtrue_response["commit"]["commit"]["verification"]["signature"]
    )
    assert (
        obj.commit.commit.verification.payload
        == branch_fixtrue_response["commit"]["commit"]["verification"]["payload"]
    )
    assert (
        obj.commit.author.gravatar_id == branch_fixtrue_response["commit"]["author"]["gravatar_id"]
    )
    assert obj.commit.author.avatar_url == branch_fixtrue_response["commit"]["author"]["avatar_url"]
    assert obj.commit.author.url == branch_fixtrue_response["commit"]["author"]["url"]
    assert obj.commit.author.id == branch_fixtrue_response["commit"]["author"]["id"]
    assert obj.commit.author.login == branch_fixtrue_response["commit"]["author"]["login"]
    assert obj.commit.parents[0].sha == branch_fixtrue_response["commit"]["parents"][0]["sha"]
    assert obj.commit.parents[0].url == branch_fixtrue_response["commit"]["parents"][0]["url"]
    assert obj.commit.url == branch_fixtrue_response["commit"]["url"]
    assert (
        obj.commit.committer.gravatar_id
        == branch_fixtrue_response["commit"]["committer"]["gravatar_id"]
    )
    assert (
        obj.commit.committer.avatar_url
        == branch_fixtrue_response["commit"]["committer"]["avatar_url"]
    )
    assert obj.commit.committer.url == branch_fixtrue_response["commit"]["committer"]["url"]
    assert obj.commit.committer.id == branch_fixtrue_response["commit"]["committer"]["id"]
    assert obj.commit.committer.login == branch_fixtrue_response["commit"]["committer"]["login"]
    assert obj.protected == branch_fixtrue_response["protected"]
    assert obj.protection.enabled == branch_fixtrue_response["protection"]["enabled"]
    assert (
        obj.protection.required_status_checks.enforcement_level
        == branch_fixtrue_response["protection"]["required_status_checks"]["enforcement_level"]
    )
    assert (
        obj.protection.required_status_checks.contexts[0]
        == branch_fixtrue_response["protection"]["required_status_checks"]["contexts"][0]
    )
    assert obj.protection_url == branch_fixtrue_response["protection_url"]



"""
Class object for AIOGitHubAPIReposTrafficClones
Documentation: https://docs.github.com/en/rest/reference/repos#get-repository-clones

Generated by generate/generate.py - 2020-08-02 12:24:23.865974
"""

from ...base import AIOGitHubAPIBase


class Clones(AIOGitHubAPIBase):
    @property
    def timestamp(self):
        return self.attributes.get("timestamp", "")

    @property
    def count(self):
        return self.attributes.get("count", None)

    @property
    def uniques(self):
        return self.attributes.get("uniques", None)


class AIOGitHubAPIReposTrafficClones(AIOGitHubAPIBase):
    @property
    def count(self):
        return self.attributes.get("count", None)

    @property
    def uniques(self):
        return self.attributes.get("uniques", None)

    @property
    def clones(self):
        return [Clones(x) for x in self.attributes.get("clones", [])]



"""
Class object for AIOGitHubAPIOrgsTeam
Documentation: https://docs.github.com/en/rest/reference/teams#get-a-team-by-name

Generated by generate/generate.py - 2020-08-02 10:33:09.248066
"""

from ..base import AIOGitHubAPIBase


class Organization(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def hooks_url(self):
        return self.attributes.get("hooks_url", "")

    @property
    def issues_url(self):
        return self.attributes.get("issues_url", "")

    @property
    def members_url(self):
        return self.attributes.get("members_url", "")

    @property
    def public_members_url(self):
        return self.attributes.get("public_members_url", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def company(self):
        return self.attributes.get("company", "")

    @property
    def blog(self):
        return self.attributes.get("blog", "")

    @property
    def location(self):
        return self.attributes.get("location", "")

    @property
    def email(self):
        return self.attributes.get("email", "")

    @property
    def twitter_username(self):
        return self.attributes.get("twitter_username", "")

    @property
    def is_verified(self):
        return self.attributes.get("is_verified", True)

    @property
    def has_organization_projects(self):
        return self.attributes.get("has_organization_projects", True)

    @property
    def has_repository_projects(self):
        return self.attributes.get("has_repository_projects", True)

    @property
    def public_repos(self):
        return self.attributes.get("public_repos", None)

    @property
    def public_gists(self):
        return self.attributes.get("public_gists", None)

    @property
    def followers(self):
        return self.attributes.get("followers", None)

    @property
    def following(self):
        return self.attributes.get("following", None)

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def type(self):
        return self.attributes.get("type", "")


class AIOGitHubAPIOrgsTeam(AIOGitHubAPIBase):
    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def slug(self):
        return self.attributes.get("slug", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def privacy(self):
        return self.attributes.get("privacy", "")

    @property
    def permission(self):
        return self.attributes.get("permission", "")

    @property
    def members_url(self):
        return self.attributes.get("members_url", "")

    @property
    def repositories_url(self):
        return self.attributes.get("repositories_url", "")

    @property
    def parent(self):
        return self.attributes.get("parent", None)

    @property
    def members_count(self):
        return self.attributes.get("members_count", None)

    @property
    def repos_count(self):
        return self.attributes.get("repos_count", None)

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at", "")

    @property
    def organization(self):
        return Organization(self.attributes.get("organization", {}))



"""Test generic api call."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI, GitHubRequestKwarg

from tests.common import MockedRequests


@pytest.mark.asyncio
async def test_generic_get(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.generic("/generic")
    assert response.status == 200
    assert response.data == {}
    assert mock_requests.called == 1
    assert mock_requests.last_request["method"] == "get"
    assert mock_requests.last_request["url"] == "https://api.github.com/generic"


@pytest.mark.asyncio
async def test_generic_post(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.generic("/generic", **{GitHubRequestKwarg.METHOD: "POST"})
    assert response.status == 200
    assert response.data == {}
    assert mock_requests.called == 1
    assert mock_requests.last_request["method"] == "post"
    assert mock_requests.last_request["url"] == "https://api.github.com/generic"
    assert mock_requests.last_request["data"] is None
    assert "json" not in mock_requests.last_request


@pytest.mark.asyncio
async def test_generic_post_with_data(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.generic(
        "/generic", {"test": "data"}, **{GitHubRequestKwarg.METHOD: "POST"}
    )
    assert response.status == 200
    assert response.data == {}
    assert mock_requests.called == 1
    assert mock_requests.last_request["method"] == "post"
    assert mock_requests.last_request["url"] == "https://api.github.com/generic"
    assert mock_requests.last_request["json"] == {"test": "data"}



"""Fixtures."""
# pylint: disable=missing-docstring
import pytest

ISSUE = {
    "id": 1,
    "node_id": "MDU6SXNzdWUx",
    "url": "https://api.github.com/repos/octocat/Hello-World/issues/1347",
    "repository_url": "https://api.github.com/repos/octocat/Hello-World",
    "labels_url": "https://api.github.com/repos/octocat/Hello-World/issues/1347/labels{/name}",
    "comments_url": "https://api.github.com/repos/octocat/Hello-World/issues/1347/comments",
    "events_url": "https://api.github.com/repos/octocat/Hello-World/issues/1347/events",
    "html_url": "https://github.com/octocat/Hello-World/issues/1347",
    "number": 1347,
    "state": "open",
    "title": "Found a bug",
    "body": "I'm having a problem with this.",
    "user": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
    "labels": [
        {
            "id": 208045946,
            "node_id": "MDU6TGFiZWwyMDgwNDU5NDY=",
            "url": "https://api.github.com/repos/octocat/Hello-World/labels/bug",
            "name": "bug",
            "description": "Something isn't working",
            "color": "f29513",
            "default": True,
        }
    ],
    "assignee": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
    "assignees": [
        {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        }
    ],
    "milestone": {
        "url": "https://api.github.com/repos/octocat/Hello-World/milestones/1",
        "html_url": "https://github.com/octocat/Hello-World/milestones/v1.0",
        "labels_url": "https://api.github.com/repos/octocat/Hello-World/milestones/1/labels",
        "id": 1002604,
        "node_id": "MDk6TWlsZXN0b25lMTAwMjYwNA==",
        "number": 1,
        "state": "open",
        "title": "v1.0",
        "description": "Tracking milestone for version 1.0",
        "creator": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "open_issues": 4,
        "closed_issues": 8,
        "created_at": "2011-04-10T20:09:31Z",
        "updated_at": "2014-03-03T18:58:10Z",
        "closed_at": "2013-02-12T13:22:01Z",
        "due_on": "2012-10-09T23:39:01Z",
    },
    "locked": True,
    "active_lock_reason": "too heated",
    "comments": 0,
    "pull_request": {
        "url": "https://api.github.com/repos/octocat/Hello-World/pulls/1347",
        "html_url": "https://github.com/octocat/Hello-World/pull/1347",
        "diff_url": "https://github.com/octocat/Hello-World/pull/1347.diff",
        "patch_url": "https://github.com/octocat/Hello-World/pull/1347.patch",
    },
    "closed_at": None,
    "created_at": "2011-04-22T13:33:48Z",
    "updated_at": "2011-04-22T13:33:48Z",
    "closed_by": {
        "login": "octocat",
        "id": 1,
        "node_id": "MDQ6VXNlcjE=",
        "avatar_url": "https://github.com/images/error/octocat_happy.gif",
        "gravatar_id": "",
        "url": "https://api.github.com/users/octocat",
        "html_url": "https://github.com/octocat",
        "followers_url": "https://api.github.com/users/octocat/followers",
        "following_url": "https://api.github.com/users/octocat/following{/other_user}",
        "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
        "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
        "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
        "organizations_url": "https://api.github.com/users/octocat/orgs",
        "repos_url": "https://api.github.com/users/octocat/repos",
        "events_url": "https://api.github.com/users/octocat/events{/privacy}",
        "received_events_url": "https://api.github.com/users/octocat/received_events",
        "type": "User",
        "site_admin": False,
    },
}


@pytest.fixture()
def issue_response():
    return ISSUE


@pytest.fixture()
def issues_response():
    return [ISSUE]



"""
Methods for the events namespace

https://docs.github.com/en/rest/reference/activity#events
"""

from __future__ import annotations

import asyncio
from datetime import datetime, timezone
from typing import TYPE_CHECKING, Any, Awaitable, Callable, Dict, Literal, TypedDict
from uuid import uuid4

from ..const import LOGGER, GitHubRequestKwarg, RepositoryType
from ..exceptions import (
    GitHubAuthenticationException,
    GitHubException,
    GitHubNotFoundException,
    GitHubNotModifiedException,
    GitHubPermissionException,
)
from ..helpers import repository_full_name
from ..models.events import GitHubEventModel
from .base import BaseNamespace

if TYPE_CHECKING:
    from ..client import GitHubClient

_DEFAULT_BACKOFF = 300
_DEFAULT_POLL = 60


class _GitHubEventsBaseNamespace(BaseNamespace):
    """Methods for the events namespace"""

    def __init__(
        self,
        client: GitHubClient,
        space: Literal["users"] | Literal["repos"] | Literal["orgs"],
    ) -> None:
        super().__init__(client)
        self._space = space
        self._subscriptions: Dict[str, asyncio.Task[None]] = {}

    @staticmethod
    async def _wait(wait_time: float) -> None:
        """Wait for x seconds"""
        await asyncio.sleep(wait_time)

    async def subscribe(
        self,
        name: str,
        event_callback: Callable[[GitHubEventModel], Awaitable[None]],
        *,
        error_callback: Callable[[GitHubException], Awaitable[None]] | None = None,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> str:
        """
         Subscribe to an event stream.
         This returns an ID you can use with the unsubscribe method to stop listening for events.

         **Arguments**:

         `name`

         The name to return evets from, example "octocat/hello-world"

         `event_callback`

         An async function that will be called when new events come in,
         the event will be passed as the first argument.

         `error_callback` (Optional)

         An async function that will be called when errors occur,
         the exception that where raised will be passed.

        https://docs.github.com/en/rest/reference/activity#list-public-events
        """
        subscription_id = str(uuid4())

        async def _subscriber():
            _last_etag: str | None = None
            _poll_time: int = 60
            _target_time = datetime.now(tz=timezone.utc).replace(tzinfo=None).isoformat()
            LOGGER.debug("Starting event subscription for github.com/%s", name)
            subscription_task = self._subscriptions[subscription_id]
            while not subscription_task.cancelled():
                try:
                    response = await self._client.async_call_api(
                        endpoint=f"/{self._space}/{name}/events",
                        etag=_last_etag,
                        **kwargs,
                    )
                except GitHubNotModifiedException:
                    await self._wait(_poll_time)
                    continue
                except (
                    GitHubAuthenticationException,
                    GitHubNotFoundException,
                    GitHubPermissionException,
                ) as err:
                    await error_callback(err)
                    break
                except GitHubException as err:
                    if error_callback is not None:
                        await error_callback(err)
                    await self._wait(_DEFAULT_BACKOFF)
                    continue
                else:
                    _last_etag = response.headers.etag
                    _poll_time = (
                        int(response.headers.x_poll_interval)
                        if response.headers.x_poll_interval
                        else _DEFAULT_POLL
                    )

                    response.data = [
                        GitHubEventModel(event) for event in reversed(response.data or [])
                    ]

                    for event in response.data:
                        if event.created_at < _target_time:
                            continue
                        _target_time = event.created_at

                        LOGGER.debug("New %s for %s", event.type, name)
                        try:
                            await event_callback(event)
                        except Exception as err:
                            if error_callback is not None:
                                await error_callback(GitHubException(err))

                await self._wait(_poll_time)

            LOGGER.debug("Stopping event subscription for github.com/%s", name)
            self.unsubscribe(subscription_id=subscription_id)

        self._subscriptions[subscription_id] = asyncio.create_task(_subscriber())

        return subscription_id

    def unsubscribe(
        self,
        *,
        subscription_id: str | None = None,
    ) -> list[asyncio.Task]:
        """
        Unsubscribe to an event stream

        **Arguments**:

        `subscription_id` (Optional)

        The ID you got when you subscribed, if omitted all active subscriptions will be stopped.
        """

        def _cancel_subscription(subid: str, task: asyncio.Task[None]) -> asyncio.Task:
            if not task.cancelled():
                task.cancel()
            del self._subscriptions[subid]
            return task

        return [
            _cancel_subscription(subid=subid, task=task)
            for subid in (
                [subscription_id]
                if subscription_id is not None
                else list(self._subscriptions.keys())
            )
            if (task := self._subscriptions.get(subid)) is not None
        ]

    async def unsubscribe_all(self) -> None:
        """Unsubscribe and wait for all subscriptions to be done."""
        if tasks := self.unsubscribe():
            await asyncio.wait(tasks)


class GitHubEventsReposNamespace(_GitHubEventsBaseNamespace):
    """Methods for the repository events namespace"""

    def __init__(self, client: GitHubClient) -> None:
        super().__init__(client, space="repos")

    async def subscribe(
        self,
        repository: RepositoryType,
        event_callback: Callable[[GitHubEventModel], Awaitable[None]],
        *,
        error_callback: Callable[[], Awaitable[None]] | None = None,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> str:
        """
         Subscribe to an event stream.
         This returns an ID you can use with the unsubscribe method to stop listening for events.

         **Arguments**:

         `repository`

         The repository to return evets from, example "octocat/hello-world"

         `event_callback`

         An async function that will be called when new events come in,
         the event will be passed as the first argument.

         `error_callback` (Optional)

         An async function that will be called when errors occur,
         the exception that where raised will be passed.

        https://docs.github.com/en/rest/reference/activity#list-repository-events
        """
        return await super().subscribe(
            name=repository_full_name(repository),
            event_callback=event_callback,
            error_callback=error_callback,
            **kwargs,
        )



"""
Class object for AIOGitHubAPILoginDevice
Documentation: https://docs.github.com/en/developers/apps/authorizing-oauth-apps#device-flow

Generated by generate/generate.py - 2020-09-08 17:49:51.525706
"""

from ..base import AIOGitHubAPIBase


class AIOGitHubAPILoginDevice(AIOGitHubAPIBase):
    @property
    def device_code(self):
        return self.attributes.get("device_code", "")

    @property
    def user_code(self):
        return self.attributes.get("user_code", "")

    @property
    def verification_uri(self):
        return self.attributes.get("verification_uri", "")

    @property
    def expires_in(self):
        return self.attributes.get("expires_in", None)

    @property
    def interval(self):
        return self.attributes.get("interval", None)



"""GitHub issue_comment data class."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase
from .user import GitHubUserModel


class GitHubIssueCommentModel(GitHubDataModelBase):
    """GitHub issue_comment data class."""

    author_association: str | None = None
    body: str | None = None
    created_at: str | None = None
    html_url: str | None = None
    id: int | None = None
    issue_url: str | None = None
    updated_at: str | None = None
    url: str | None = None
    user: GitHubUserModel | None = None

    def _generate_user(self, data: Dict[str, Any] | None) -> GitHubUserModel:
        """Generate GitHubUserModel."""
        return GitHubUserModel(data) if data else None



"""Test fixtures and configuration."""
# pylint: disable=redefined-outer-name,protected-access
from datetime import datetime
import logging
from unittest.mock import AsyncMock, patch

import aiohttp
import pytest
import pytest_asyncio

from aiogithubapi import GitHub, GitHubAPI, GitHubDevice, GitHubDeviceAPI
from aiogithubapi.client import AIOGitHubAPIClient
from aiogithubapi.const import PROJECT_NAME

from .common import CLIENT_ID, TOKEN

from tests.common import MockedRequests, MockResponse

logging.basicConfig(level=logging.ERROR)
logging.getLogger(PROJECT_NAME).setLevel(logging.DEBUG)


@pytest.fixture()
def asyncio_sleep():
    """Mock asyncio.sleep."""
    with patch("asyncio.sleep", new=AsyncMock()) as mock_sleep:
        yield mock_sleep


@pytest.fixture()
def mock_requests():
    """Return a new mock request instance."""
    yield MockedRequests()


@pytest.fixture()
def response():
    """Return a new mock response instance."""
    yield MockResponse()


@pytest.fixture()
def mock_response():
    """Return a new mock response instance."""
    yield MockResponse()


@pytest_asyncio.fixture(autouse=True)
async def client_session(mock_response, mock_requests):
    """Mock our the request part of the client session."""

    async def _mocked_request(*args, **kwargs):
        if len(args) > 2:
            mock_response.mock_endpoint = args[2].split(".com/")[-1]
            mock_requests.add({"method": args[1], "url": args[2], **kwargs})
        else:
            mock_response.mock_endpoint = args[1].split(".com/")[-1]
            mock_requests.add({"method": args[0], "url": args[1], **kwargs})
        return mock_response

    async with aiohttp.ClientSession() as session:
        mock_requests.clear()
        session._request = _mocked_request  # pylint: disable=protected-access
        yield session


@pytest_asyncio.fixture
async def client(client_session):
    """Fixture to provide a GitHub client object."""
    client_obj = AIOGitHubAPIClient(client_session, TOKEN)
    yield client_obj


@pytest_asyncio.fixture
async def github(client, client_session):
    """Fixture to provide a GitHub object."""
    async with GitHub(TOKEN, session=client_session) as github_obj:
        github_obj.client = client
        yield github_obj


@pytest_asyncio.fixture
async def github_api(client_session, mock_response):
    """Fixture to provide a GitHub object."""
    mock_response.throw_on_file_error = True
    async with GitHubAPI(
        token=TOKEN,
        session=client_session,
    ) as github_obj:
        yield github_obj


@pytest_asyncio.fixture
async def github_device(client_session):
    """Fixture to provide a GitHub Devlice object."""
    async with GitHubDevice(CLIENT_ID, session=client_session) as github_device_obj:
        yield github_device_obj


@pytest_asyncio.fixture
async def github_device_api(client_session):
    """Fixture to provide a GitHub Devlice object."""
    async with GitHubDeviceAPI(
        CLIENT_ID,
        session=client_session,
    ) as github_device_obj:
        github_device_obj._interval = 1
        github_device_obj._expires = datetime.timestamp(datetime.now()) + 900
        yield github_device_obj



"""Helpers for AIOGitHubAPI."""

from __future__ import annotations

from typing import TYPE_CHECKING, Optional

import aiohttp

from .const import HttpMethod, Repository, RepositoryType
from .legacy.helpers import (
    async_call_api as legacy_async_call_api,
    short_message,
    short_sha,
)
from .objects.base import AIOGitHubAPIResponse


def repository_full_name(repository: RepositoryType) -> str:
    """Return the repository name."""
    if isinstance(repository, str):
        return repository
    if isinstance(repository, Repository):
        return repository.full_name
    return f"{repository['owner']}/{repository['repo']}"


async def async_call_api(
    session: aiohttp.ClientSession,
    method: HttpMethod,
    url: str,
    headers: dict,
    params: Optional[dict] = None,
    data: dict or str or None = None,
    jsondata: bool = True,
    returnjson: bool = True,
) -> AIOGitHubAPIResponse:
    """Deprecated: Execute the API call."""
    return await legacy_async_call_api(
        session, method, url, headers, params, data, jsondata, returnjson
    )



# pylint: disable=missing-docstring, redefined-outer-name, unused-import
import datetime
import json

import pytest

from aiogithubapi import AIOGitHubAPINotModifiedException, GitHub

from tests.common import TOKEN
from tests.legacy.responses.repos.traffic.clones_fixtrue import clones_fixtrue_response
from tests.legacy.responses.repos.traffic.pageviews_fixtrue import (
    pageviews_fixtrue_response,
)
from tests.legacy.responses.repository_fixture import repository_response


@pytest.mark.asyncio
async def test_get_clones(aresponses, repository_response, clones_fixtrue_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/traffic/clones",
        "get",
        aresponses.Response(
            text=json.dumps(clones_fixtrue_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/traffic/clones",
        "get",
        aresponses.Response(status=304),
    )

    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        clones = await repository.traffic.get_clones()
        assert clones.count == 173

        with pytest.raises(AIOGitHubAPINotModifiedException):
            await repository.traffic.get_clones(etag=github.client.last_response.etag)


@pytest.mark.asyncio
async def test_get_views(aresponses, repository_response, pageviews_fixtrue_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/traffic/views",
        "get",
        aresponses.Response(
            text=json.dumps(pageviews_fixtrue_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/traffic/views",
        "get",
        aresponses.Response(status=304),
    )

    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        views = await repository.traffic.get_views()
        assert views.count == 14850

        with pytest.raises(AIOGitHubAPINotModifiedException):
            await repository.traffic.get_views(etag=github.client.last_response.etag)



"""
Class object for AIOGitHubAPIReposLabel
Documentation: https://docs.github.com/en/rest/reference/issues#get-a-label

Generated by generate/generate.py - 2020-08-02 10:32:07.005845
"""

from ..base import AIOGitHubAPIBase


class AIOGitHubAPIReposLabel(AIOGitHubAPIBase):
    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def color(self):
        return self.attributes.get("color", "")

    @property
    def default(self):
        return self.attributes.get("default", True)



"""GitHub git data class."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase


class GitHubGitTreeEntryModel(GitHubDataModelBase):
    """GitHub git tree entry model."""

    mode: str | None = None
    path: str | None = None
    sha: str | None = None
    size: int | None = None
    type: str | None = None
    url: str | None = None


class GitHubGitTreeModel(GitHubDataModelBase):
    """GitHub git data class."""

    sha: str | None = None
    tree: list[GitHubGitTreeEntryModel] | None = None
    truncated: bool | None = None
    url: str | None = None

    def _generate_tree(self, data: list[Dict[str, Any]]) -> list:
        """Generate tree entries."""
        return [GitHubGitTreeEntryModel(entry) for entry in data or []]



"""Fixtures."""
# pylint: disable=missing-docstring
import pytest


@pytest.fixture()
def issue_comments_response():
    return [
        {
            "id": 1,
            "node_id": "MDEyOklzc3VlQ29tbWVudDE=",
            "url": "https://api.github.com/repos/octocat/Hello-World/issues/comments/1",
            "html_url": "https://github.com/octocat/Hello-World/issues/1347#issuecomment-1",
            "body": "Me too",
            "user": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "created_at": "2011-04-14T16:00:49Z",
            "updated_at": "2011-04-14T16:00:49Z",
        }
    ]



"""
Methods for the traffic namespace

https://docs.github.com/en/rest/reference/repos#traffic
"""

from __future__ import annotations

from typing import Any, Dict

from ..const import GitHubRequestKwarg, RepositoryType
from ..models.clones import GitHubClonesModel
from ..models.response import GitHubResponseModel
from ..models.views import GitHubViewsModel
from .base import BaseNamespace


class GitHubTrafficNamespace(BaseNamespace):
    """Methods for the traffic namespace"""

    async def clones(
        self,
        repository: RepositoryType,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubClonesModel]:
        """
        Get repository clones

        Get the total number of clones and breakdown per day or week for the last 14 days.
        Timestamps are aligned to UTC midnight of the beginning of the day or week.
        Week begins on Monday.

         **Arguments**:

         `repository`

         The repository to return clones for, example "octocat/hello-world"

        https://docs.github.com/en/rest/reference/git#get-a-tree
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository}/traffic/clones",
            **kwargs,
        )
        response.data = GitHubClonesModel(response.data)
        return response

    async def views(
        self,
        repository: RepositoryType,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubViewsModel]:
        """
        Get page views

        Get the total number of views and breakdown per day or week for the last 14 days.
        Timestamps are aligned to UTC midnight of the beginning of the day or week.
        Week begins on Monday.

         **Arguments**:

         `repository`

         The repository to return views for, example "octocat/hello-world"

        https://docs.github.com/en/rest/reference/git#get-a-tree
        """
        response = await self._client.async_call_api(
            endpoint=f"/repos/{repository}/traffic/views",
            **kwargs,
        )
        response.data = GitHubViewsModel(response.data)
        return response



"""
Methods for the orgs namespace

https://docs.github.com/en/rest/reference/orgs
"""

from __future__ import annotations

from typing import Any, Dict

from ..const import GitHubRequestKwarg, HttpMethod
from ..models.organization import (
    GitHubOrganizationMinimalModel,
    GitHubOrganizationModel,
)
from ..models.response import GitHubResponseModel
from .base import BaseNamespace
from .projects import GitHubOrganizationProjectsNamespace


class GitHubOrgsNamespace(BaseNamespace):
    """Methods for the orgs namespace"""

    def __post_init__(self) -> None:
        self._projects = GitHubOrganizationProjectsNamespace(self._client)

    @property
    def projects(self) -> GitHubOrganizationProjectsNamespace:
        """Property to access the users projects namespace"""
        return self._projects

    async def list(
        self,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[list[GitHubOrganizationMinimalModel]]:
        """
         List organizations


        https://docs.github.com/en/rest/reference/orgs#list-organizations
        """
        response = await self._client.async_call_api(
            endpoint="/organizations",
            **kwargs,
        )
        response.data = [GitHubOrganizationMinimalModel(org) for org in response.data or []]
        return response

    async def get(
        self,
        org: str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubOrganizationModel]:
        """
         Get a organization

         **Arguments**:

         `org`

         The organization to return, example "octocat"

        https://docs.github.com/en/rest/reference/orgs#get-an-organization
        """
        response = await self._client.async_call_api(
            endpoint=f"/orgs/{org}",
            **kwargs,
        )
        response.data = GitHubOrganizationModel(response.data)
        return response

    async def update(
        self,
        org: str,
        data: dict[str, Any],
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubOrganizationModel]:
        """
         Update an organization

         **Arguments**:

         `org`

         The organization to update, example "octocat"

         `data`

         A dictionary of data to update.

        https://docs.github.com/en/rest/reference/orgs#update-an-organization
        """
        response = await self._client.async_call_api(
            endpoint=f"/orgs/{org}",
            data=data,
            method=HttpMethod.PATCH,
            **kwargs,
        )
        response.data = GitHubOrganizationModel(response.data)
        return response



"""GitHub issue data class."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase
from .label import GitHubLabelModel
from .milestone import GitHubMilestoneModel
from .user import GitHubUserModel


class _PullRequest(GitHubDataModelBase):
    """GitHub pull_request data model."""

    url: str | None = None
    html_url: str | None = None
    diff_url: str | None = None
    patch_url: str | None = None


class GitHubIssueModel(GitHubDataModelBase):
    """GitHub issue data class."""

    active_lock_reason: str | None = None
    author_association: str | None = None
    body: str | None = None
    closed_at: str | None = None
    closed_by: GitHubUserModel | None = None
    comments_url: str | None = None
    comments: int | None = None
    created_at: str | None = None
    events_url: str | None = None
    html_url: str | None = None
    id: int | None = None
    labels_url: str | None = None
    locked: bool | None = None
    milestone: GitHubMilestoneModel | None = None
    number: int | None = None
    repository_url: str | None = None
    state: str | None = None
    title: str | None = None
    updated_at: str | None = None
    url: str | None = None
    pull_request: _PullRequest | None = None
    assignees: list[GitHubUserModel] | None = None
    assignee: GitHubUserModel | None = None
    user: GitHubUserModel | None = None
    labels: list[GitHubLabelModel] | None = None

    @property
    def is_pull_request(self) -> bool:
        """Return True if issue is a pull request."""
        return self.pull_request is not None

    def _generate_labels(self, data: list[Dict[str, Any]]) -> list[GitHubLabelModel]:
        """Generate GitHubLabelModel list from data."""
        return [GitHubLabelModel(label) for label in data or []]

    def _generate_assignees(self, data: list[Dict[str, Any]]) -> list[GitHubUserModel]:
        """Generate GitHubUserModel list from data."""
        return [GitHubUserModel(user) for user in data or []]

    def _generate_assignee(self, data: Dict[str, Any] | None) -> GitHubUserModel:
        """Generate GitHubUserModel from data."""
        return GitHubUserModel(data) if data else None

    def _generate_milestone(self, data: Dict[str, Any] | None) -> "GitHubMilestoneModel":
        """Generate GitHubMilestoneModel from data."""
        return GitHubMilestoneModel(data) if data else None

    def _generate_closed_by(self, data: Dict[str, Any] | None) -> GitHubUserModel:
        """Generate GitHubUserModel from data."""
        return GitHubUserModel(data) if data else None

    def _generate_pull_request(self, data: Dict[str, Any] | None) -> _PullRequest:
        """Generate GitHubPullRequestModel from data."""
        return _PullRequest(data) if data else None



"""Test orgs namespace."""
# pylint: disable=missing-docstring

import pytest

from aiogithubapi import (
    GitHubAPI,
    GitHubOrganizationMinimalModel,
    GitHubOrganizationModel,
)

from tests.common import TEST_ORGANIZATION, MockedRequests


@pytest.mark.asyncio
async def test_list_organizations(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.orgs.list()
    assert response.status == 200
    assert isinstance(response.data, list)
    fist = response.data[0]
    assert isinstance(fist, GitHubOrganizationMinimalModel)
    assert fist.login == "github"
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/organizations"


@pytest.mark.asyncio
async def test_get_organization(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.orgs.get(TEST_ORGANIZATION)
    assert response.status == 200
    assert isinstance(response.data, GitHubOrganizationModel)
    assert response.data.name == "github"
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/orgs/octocat"


@pytest.mark.asyncio
async def test_update_organization(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.orgs.update(TEST_ORGANIZATION, {"test": "data"})
    assert response.status == 200
    assert isinstance(response.data, GitHubOrganizationModel)
    assert response.data.name == "github"
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/orgs/octocat"



# pylint: disable=missing-docstring, redefined-outer-name, unused-import
import pytest

from aiogithubapi import GitHub

from tests.common import TOKEN, load_fixture


@pytest.mark.asyncio
async def test_graphql(client_session):
    async with GitHub(TOKEN, session=client_session) as github:
        repository = await github.graphql(
            query='query{repository(owner: "awesome", name: "repo"){description}}'
        )
        assert repository["repository"]["description"] == "This your first repo!"



"""AIOGitHubAPI: AIOGitHubAPI"""

from __future__ import annotations

import os
from typing import Any, Dict

import aiohttp

from .client import GitHubClient
from .const import GitHubClientKwarg, GitHubRequestKwarg, HttpMethod, RepositoryType
from .legacy.github import AIOGitHubAPI as LegacyAIOGitHubAPI
from .models.base import GitHubBase
from .models.meta import GitHubMetaModel
from .models.rate_limit import GitHubRateLimitModel
from .models.response import GitHubResponseModel
from .namespaces.notifications import GitHubNotificationsNamespace
from .namespaces.orgs import GitHubOrgsNamespace
from .namespaces.projects import GitHubBaseProjectsNamespace
from .namespaces.repos import GitHubReposNamespace
from .namespaces.user import GitHubUserNamespace
from .namespaces.users import GitHubUsersNamespace


class AIOGitHubAPI(LegacyAIOGitHubAPI):
    """Dummy class to not break existing code."""


class GitHub(GitHubBase):
    """
    GitHub

    This is the main class this is where it all starts.
    """

    _close_session = False

    def __init__(
        self,
        token: str = None,
        session: aiohttp.ClientSession = None,
        *,
        api_version: str | None = None,
        **kwargs: Dict[GitHubClientKwarg, Any],
    ) -> None:
        """
        Initialise GitHub API client.

        **Arguments**:

        `token` (Optional)

        Your GitHub [Personal Access Token](https://github.com/settings/tokens).
        Defaults to getting the token from the GITHUB_TOKEN env variable.

        `session` (Optional)

        `aiohttp.ClientSession` to be used by this package.
        If you do not pass one, one will be created for you.

        `**kwargs` (Optional)

        Pass additional arguments.
        See the `aiogithubapi.const.GitHubClientKwarg` enum for valid options.
        """
        if session is None:
            session = aiohttp.ClientSession()
            self._close_session = True

        if token is None:
            token = os.getenv("GITHUB_TOKEN")

        self._session = session
        self._client = GitHubClient(token=token, session=session, api_version=api_version, **kwargs)

        # Namespaces
        self._notifications = GitHubNotificationsNamespace(self._client)
        self._repos = GitHubReposNamespace(self._client)
        self._user = GitHubUserNamespace(self._client)
        self._users = GitHubUsersNamespace(self._client)
        self._orgs = GitHubOrgsNamespace(self._client)
        self._projects = GitHubBaseProjectsNamespace(self._client)

    @property
    def notifications(self) -> GitHubNotificationsNamespace:
        """Property to access the notifications namespace."""
        return self._notifications

    @property
    def repos(self) -> GitHubReposNamespace:
        """Property to access the repos namespace."""
        return self._repos

    @property
    def users(self) -> GitHubUsersNamespace:
        """Property to access the users namespace."""
        return self._users

    @property
    def user(self) -> GitHubUserNamespace:
        """Property to access the user namespace."""
        return self._user

    @property
    def orgs(self) -> GitHubOrgsNamespace:
        """Property to access the orgs namespace."""
        return self._orgs

    @property
    def projects(self) -> GitHubBaseProjectsNamespace:
        """Property to access the base projects namespace."""
        return self._projects

    async def __aenter__(self) -> GitHub:
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info) -> None:
        """Async exit."""
        await self.repos.events.unsubscribe_all()
        await self.close_session()

    async def close_session(self) -> None:
        """Close open client session."""
        if self._session and self._close_session:
            await self._session.close()

    async def generic(
        self,
        endpoint: str,
        data: Dict[str, Any] | str | None = None,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[Any]:
        """
         Generic REST API call

         This can be used to call any API endpoint.
         Usefull when you need something that does not have a dedicated method.

         Returns a `aiogithubapi.models.response.GitHubResponseModel` object,
         where the data attribute contains the non-typed response.

         **Arguments**:

         `endpoint`

         The API endpoint to call.
        https://docs.github.com/en/rest
        """
        return await self._client.async_call_api(endpoint=endpoint, data=data, **kwargs)

    async def emojis(
        self,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[Dict[str, str]]:
        """
        Get emojis
        Lists all the emojis available to use on GitHub.

        https://docs.github.com/en/rest/reference/emojis#get-emojis
        """
        return await self._client.async_call_api(endpoint="/emojis", **kwargs)

    async def versions(
        self,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[list[str]]:
        """
        Get all supported GitHub API versions.

        https://docs.github.com/en/rest/meta#get-all-api-versions
        """
        return await self._client.async_call_api(endpoint="/versions", **kwargs)

    async def markdown(
        self,
        text: str,
        *,
        mode: str | None = None,
        context: RepositoryType | None = None,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[str]:
        """
         Render a Markdown document

         **Arguments**:

         `text`

         The Markdown text to render in HTML.

         `mode`

         The rendering mode to use. Defaults to 'markdown'.
         If context is provided this will be set to 'gfm'.

         `context`

         The repository context to render the Markdown in.

        https://docs.github.com/en/rest/reference/markdown#render-a-markdown-document
        """
        return await self._client.async_call_api(
            endpoint="/markdown",
            data={"text": text, "mode": "gfm" if context else mode, "context": context},
            **{**kwargs, GitHubRequestKwarg.METHOD: HttpMethod.POST},
        )

    async def meta(
        self,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubMetaModel]:
        """
        Returns meta information about GitHub, including a list of GitHub's IP addresses.

        https://docs.github.com/en/rest/reference/meta#get-github-meta-information
        """
        response = await self._client.async_call_api(endpoint="/meta", **kwargs)
        response.data = GitHubMetaModel(response.data)
        return response

    async def zen(self, **kwargs: Dict[GitHubRequestKwarg, Any]) -> GitHubResponseModel[str]:
        """
        ZEN

        https://docs.github.com/en/rest/guides/getting-started-with-the-rest-api#hello-world
        """
        return await self._client.async_call_api(endpoint="/zen", **kwargs)

    async def octocat(self, **kwargs: Dict[GitHubRequestKwarg, Any]) -> GitHubResponseModel[str]:
        """
        Get the octocat as ASCII art

        https://docs.github.com/en/rest/reference/meta#get-octocat
        """
        return await self._client.async_call_api(endpoint="/octocat", **kwargs)

    async def rate_limit(
        self,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubRateLimitModel]:
        """
        Get rate limit status for the authenticated user

        https://docs.github.com/en/rest/reference/rate-limit#get-rate-limit-status-for-the-authenticated-user
        """
        response = await self._client.async_call_api(endpoint="/rate_limit", **kwargs)
        response.data = GitHubRateLimitModel(response.data)
        return response

    async def graphql(
        self,
        query: str,
        variables: Dict[str, Any] | None = None,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[dict]:
        """
        Use the GitHub GraphQL API

        https://docs.github.com/en/graphql
        """
        return await self._client.async_call_api(
            endpoint="/graphql",
            data={"query": query, "variables": variables or {}},
            method=HttpMethod.POST,
            **kwargs,
        )



"""GitHub views data class."""

from __future__ import annotations

from typing import Any, Dict

from .base import GitHubDataModelBase


class _View(GitHubDataModelBase):
    """GitHub view data."""

    timestamp: str | None = None
    count: int | None = None
    uniques: int | None = None


class GitHubViewsModel(GitHubDataModelBase):
    """GitHub views data class."""

    count: int | None = None
    uniques: int | None = None
    views: list[_View] | None = None

    def _generate_views(self, data: list[Dict[str, Any]]) -> list[_View]:
        """Generate views from list of dicts."""
        return [_View(view) for view in data or []]



"""Construction tests for the main class."""
# pylint: disable=protected-access,missing-function-docstring
from __future__ import annotations

from unittest.mock import patch

import aiohttp
import pytest

from aiogithubapi import GitHubAPI

from tests.common import TOKEN


@pytest.mark.asyncio
async def test_session_creation():
    github = GitHubAPI()
    assert github._session
    assert isinstance(github._session, aiohttp.ClientSession)

    assert not github._session.closed
    await github.close_session()
    assert github._session.closed


@pytest.mark.asyncio
async def test_session_pass():
    async with aiohttp.ClientSession() as session:
        github = GitHubAPI(session=session)
        assert github._session is session


@pytest.mark.asyncio
async def test_session_creation_with_enter():
    async with GitHubAPI() as github:
        assert github._session
        assert isinstance(github._session, aiohttp.ClientSession)
        assert not github._session.closed

    assert github._session.closed


@pytest.mark.asyncio
async def test_token():
    assert GitHubAPI()._client._base_request_data.token is None
    assert GitHubAPI(token=TOKEN)._client._base_request_data.token == TOKEN
    with patch("os.environ", {"GITHUB_TOKEN": TOKEN}):
        assert GitHubAPI()._client._base_request_data.token == TOKEN


@pytest.mark.asyncio
async def test_api_version():
    assert GitHubAPI()._client._base_request_data.token is None
    assert GitHubAPI(token=TOKEN, api_version="3000-01-01")._client._base_request_data.api_version == "3000-01-01"



"""GitHub device login data class."""

from __future__ import annotations

from .base import GitHubDataModelBase


class GitHubLoginDeviceModel(GitHubDataModelBase):
    """GitHub device login data class."""

    device_code: str | None = None
    user_code: str | None = None
    verification_uri: str | None = None
    expires_in: int | None = None
    interval: int | None = None



"""
Methods for the users namespace

https://docs.github.com/en/rest/reference/users
"""

from __future__ import annotations

from typing import Any, Dict

from ..const import GitHubRequestKwarg
from ..models.organization import GitHubOrganizationMinimalModel
from ..models.repository import GitHubRepositoryModel
from ..models.response import GitHubResponseModel
from ..models.user import GitHubUserModel
from .base import BaseNamespace
from .projects import GitHubUsersProjectsNamespace


class GitHubUsersNamespace(BaseNamespace):
    """Methods for the users namespace"""

    def __post_init__(self) -> None:
        self._projects = GitHubUsersProjectsNamespace(self._client)

    @property
    def projects(self) -> GitHubUsersProjectsNamespace:
        """Property to access the users projects namespace"""
        return self._projects

    async def get(
        self,
        username: str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[GitHubUserModel]:
        """
         Get a user

         **Arguments**:

         `username`

         The username to return, example "octocat"

        https://docs.github.com/en/rest/reference/users#get-a-user
        """
        response = await self._client.async_call_api(
            endpoint=f"/users/{username}",
            **kwargs,
        )
        response.data = GitHubUserModel(response.data)

        return response

    async def starred(
        self,
        username: str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[list[GitHubRepositoryModel]]:
        """
         Get the starred repositories of a user

         **Arguments**:

         `username`

         The username to return, example "octocat"


        https://docs.github.com/en/rest/reference/users#get-a-user
        """
        response = await self._client.async_call_api(
            endpoint=f"/users/{username}/starred",
            **kwargs,
        )

        response.data = [GitHubRepositoryModel(data) for data in response.data]

        return response

    async def repos(
        self,
        username: str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[list[GitHubRepositoryModel]]:
        """
         Get the repositories of a user

         **Arguments**:

         `username`

         The username to return, example "octocat"


        https://docs.github.com/en/rest/reference/repos#list-repositories-for-a-user
        """
        response = await self._client.async_call_api(
            endpoint=f"/users/{username}/repos",
            **kwargs,
        )

        response.data = [GitHubRepositoryModel(data) for data in response.data]

        return response

    async def orgs(
        self,
        username: str,
        **kwargs: Dict[GitHubRequestKwarg, Any],
    ) -> GitHubResponseModel[list[GitHubOrganizationMinimalModel]]:
        """
         List public organization memberships for the specified user.

         **Arguments**:

         `username`

         The username to return, example "octocat"


        https://docs.github.com/en/rest/reference/orgs#list-organizations-for-a-user
        """
        response = await self._client.async_call_api(
            endpoint=f"/users/{username}/orgs",
            **kwargs,
        )

        response.data = [GitHubOrganizationMinimalModel(data) for data in response.data or []]

        return response



"""Test meta."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI

from tests.common import MockedRequests


@pytest.mark.asyncio
async def test_meta(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.meta()
    assert response.status == 200
    assert response.data.verifiable_password_authentication
    assert (
        response.data.ssh_key_fingerprints.sha256_ed25519
        == "+DiY3wvvV6TuJJhbpZisF/zLDA0zPMSvHdkr4UvCOqU"
    )
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/meta"



"""GitHub contents data class."""

from __future__ import annotations

from .base import GitHubDataModelBase


class GitHubContentsModel(GitHubDataModelBase):
    """GitHub contents data class."""

    type: str | None = None
    encoding: str | None = None
    size: int | None = None
    name: str | None = None
    path: str | None = None
    content: str | None = None
    sha: str | None = None
    url: str | None = None
    git_url: str | None = None
    html_url: str | None = None
    download_url: str | None = None



"""Test generic api call."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI

from tests.common import (
    HEADERS_TEXT,
    TEST_REPOSITORY_NAME,
    MockedRequests,
    MockResponse,
)


@pytest.mark.asyncio
async def test_markdown_base(
    github_api: GitHubAPI,
    mock_requests: MockedRequests,
    mock_response: MockResponse,
):
    mock_response.mock_headers = HEADERS_TEXT
    response = await github_api.markdown("Lorem ipsum...")
    assert response.status == 200
    assert response.data == "Lorem ipsum..."
    assert mock_requests.called == 1
    assert mock_requests.last_request["method"] == "post"
    assert mock_requests.last_request["url"] == "https://api.github.com/markdown"
    assert mock_requests.last_request["json"] == {
        "context": None,
        "mode": None,
        "text": "Lorem ipsum...",
    }


@pytest.mark.asyncio
async def test_markdown_mode(
    github_api: GitHubAPI,
    mock_requests: MockedRequests,
    mock_response: MockResponse,
):
    mock_response.mock_headers = HEADERS_TEXT
    response = await github_api.markdown("Lorem ipsum...", mode="gfm")
    assert response.status == 200
    assert response.data == "Lorem ipsum..."
    assert mock_requests.called == 1
    assert mock_requests.last_request["method"] == "post"
    assert mock_requests.last_request["url"] == "https://api.github.com/markdown"
    assert mock_requests.last_request["json"] == {
        "context": None,
        "mode": "gfm",
        "text": "Lorem ipsum...",
    }


@pytest.mark.asyncio
async def test_markdown_context(
    github_api: GitHubAPI,
    mock_requests: MockedRequests,
    mock_response: MockResponse,
):
    mock_response.mock_headers = HEADERS_TEXT
    response = await github_api.markdown("Lorem ipsum...", context=TEST_REPOSITORY_NAME)
    assert response.status == 200
    assert response.data == "Lorem ipsum..."
    assert mock_requests.called == 1
    assert mock_requests.last_request["method"] == "post"
    assert mock_requests.last_request["url"] == "https://api.github.com/markdown"
    assert mock_requests.last_request["json"] == {
        "context": TEST_REPOSITORY_NAME,
        "mode": "gfm",
        "text": "Lorem ipsum...",
    }



"""Test traffic namespace."""
# pylint: disable=missing-docstring
import pytest

from aiogithubapi import GitHubAPI

from tests.common import TEST_REPOSITORY_NAME, MockedRequests


@pytest.mark.asyncio
async def test_clones(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.repos.traffic.clones(TEST_REPOSITORY_NAME)
    assert response.status == 200
    assert len(response.data.clones) == 1
    assert response.data.clones[0].count == 1
    assert mock_requests.called == 1
    assert (
        mock_requests.last_request["url"]
        == "https://api.github.com/repos/octocat/hello-world/traffic/clones"
    )


@pytest.mark.asyncio
async def test_views(github_api: GitHubAPI, mock_requests: MockedRequests):
    response = await github_api.repos.traffic.views(TEST_REPOSITORY_NAME)
    assert response.status == 200
    assert len(response.data.views) == 1
    assert response.data.views[0].count == 1
    assert mock_requests.called == 1
    assert (
        mock_requests.last_request["url"]
        == "https://api.github.com/repos/octocat/hello-world/traffic/views"
    )



"""
Generated by generate/generate.py - 2020-08-02 10:35:28.920747
"""
from aiogithubapi.objects.users.user import AIOGitHubAPIUsersUser

from tests.legacy.responses.users.user_fixtrue import user_fixtrue_response


def test_user(user_fixtrue_response):
    obj = AIOGitHubAPIUsersUser(user_fixtrue_response)
    assert obj.login == user_fixtrue_response["login"]
    assert obj.id == user_fixtrue_response["id"]
    assert obj.node_id == user_fixtrue_response["node_id"]
    assert obj.avatar_url == user_fixtrue_response["avatar_url"]
    assert obj.gravatar_id == user_fixtrue_response["gravatar_id"]
    assert obj.url == user_fixtrue_response["url"]
    assert obj.html_url == user_fixtrue_response["html_url"]
    assert obj.followers_url == user_fixtrue_response["followers_url"]
    assert obj.following_url == user_fixtrue_response["following_url"]
    assert obj.gists_url == user_fixtrue_response["gists_url"]
    assert obj.starred_url == user_fixtrue_response["starred_url"]
    assert obj.subscriptions_url == user_fixtrue_response["subscriptions_url"]
    assert obj.organizations_url == user_fixtrue_response["organizations_url"]
    assert obj.repos_url == user_fixtrue_response["repos_url"]
    assert obj.events_url == user_fixtrue_response["events_url"]
    assert obj.received_events_url == user_fixtrue_response["received_events_url"]
    assert obj.type == user_fixtrue_response["type"]
    assert obj.site_admin == user_fixtrue_response["site_admin"]
    assert obj.name == user_fixtrue_response["name"]
    assert obj.company == user_fixtrue_response["company"]
    assert obj.blog == user_fixtrue_response["blog"]
    assert obj.location == user_fixtrue_response["location"]
    assert obj.email == user_fixtrue_response["email"]
    assert obj.hireable == user_fixtrue_response["hireable"]
    assert obj.bio == user_fixtrue_response["bio"]
    assert obj.twitter_username == user_fixtrue_response["twitter_username"]
    assert obj.public_repos == user_fixtrue_response["public_repos"]
    assert obj.public_gists == user_fixtrue_response["public_gists"]
    assert obj.followers == user_fixtrue_response["followers"]
    assert obj.following == user_fixtrue_response["following"]
    assert obj.created_at == user_fixtrue_response["created_at"]
    assert obj.updated_at == user_fixtrue_response["updated_at"]



"""
Class object for AIOGitHubAPIReposTrafficPageviews
Documentation: https://docs.github.com/en/rest/reference/repos#get-page-views

Generated by generate/generate.py - 2020-08-02 12:25:07.157407
"""

from ...base import AIOGitHubAPIBase


class Views(AIOGitHubAPIBase):
    @property
    def timestamp(self):
        return self.attributes.get("timestamp", "")

    @property
    def count(self):
        return self.attributes.get("count", None)

    @property
    def uniques(self):
        return self.attributes.get("uniques", None)


class AIOGitHubAPIReposTrafficPageviews(AIOGitHubAPIBase):
    @property
    def count(self):
        return self.attributes.get("count", None)

    @property
    def uniques(self):
        return self.attributes.get("uniques", None)

    @property
    def views(self):
        return [Views(x) for x in self.attributes.get("views", [])]



"""Test zen."""
# pylint: disable=missing-docstring
from aiohttp.hdrs import CONTENT_TYPE
import pytest

from aiogithubapi import GitHubAPI
from aiogithubapi.const import HttpContentType

from tests.common import MockedRequests, MockResponse


@pytest.mark.asyncio
async def test_zen(
    github_api: GitHubAPI,
    mock_response: MockResponse,
    mock_requests: MockedRequests,
):
    mock_response.mock_headers = {CONTENT_TYPE: HttpContentType.TEXT_PLAIN}
    response = await github_api.zen()
    assert response.status == 200
    assert response.data == "Beautiful is better than ugly."
    assert mock_requests.called == 1
    assert mock_requests.last_request["url"] == "https://api.github.com/zen"



"""GitHub meta data class."""

from __future__ import annotations

from .base import GitHubDataModelBase


class _FingerPrints(GitHubDataModelBase):
    """Representation of a GitHub tag commit."""

    sha256_rsa: str | None = None
    sha256_dsa: str | None = None
    sha256_ecdsa: str | None = None
    sha256_ed25519: str | None = None


class GitHubMetaModel(GitHubDataModelBase):
    """GitHub meta data class."""

    verifiable_password_authentication: bool | None = None
    ssh_key_fingerprints: _FingerPrints | None = None
    ssh_keys: list[str] | None = None
    hooks: list[str] | None = None
    web: list[str] | None = None
    api: list[str] | None = None
    git: list[str] | None = None
    packages: list[str] | None = None
    pages: list[str] | None = None
    importer: list[str] | None = None
    actions: list[str] | None = None
    dependabot: list[str] | None = None

    def _generate_ssh_key_fingerprints(self, data: dict) -> _FingerPrints:
        """Generate commit data."""
        return _FingerPrints(data)



# pylint: disable=missing-docstring, redefined-outer-name, unused-import
import datetime
import json

import pytest

from aiogithubapi import GitHub

from tests.common import TOKEN
from tests.legacy.responses.releases import releases_response
from tests.legacy.responses.repository_fixture import repository_response


@pytest.mark.asyncio
async def test_get_releases(aresponses, repository_response, releases_response):
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World",
        "get",
        aresponses.Response(
            text=json.dumps(repository_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/releases",
        "get",
        aresponses.Response(
            text=json.dumps(releases_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/releases",
        "get",
        aresponses.Response(
            text=json.dumps(releases_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    aresponses.add(
        "api.github.com",
        "/repos/octocat/Hello-World/releases",
        "get",
        aresponses.Response(
            text=json.dumps(releases_response),
            status=200,
            headers={
                "X-RateLimit-Remaining": "1337",
                "Content-Type": "application/json",
                "Etag": "xyz..zyx",
            },
        ),
    )
    async with GitHub(TOKEN) as github:
        repository = await github.get_repo("octocat/Hello-World")
        releases = await repository.get_releases()
        first = releases[0]
        assert first.tag_name == "v1.0.0"
        assert first.name == "v1.0.0"
        assert first.published_at == datetime.datetime(2013, 2, 27, 19, 35, 32)
        assert not first.draft
        assert not first.prerelease
        assert len(first.assets) == 1



"""
Generated by generate/generate.py - 2020-08-02 10:03:59.857088
"""
import pytest


@pytest.fixture()
def pullrequest_fixtrue_response():
    return {
        "url": "https://api.github.com/repos/octocat/Hello-World/pulls/1347",
        "id": 1,
        "node_id": "MDExOlB1bGxSZXF1ZXN0MQ==",
        "html_url": "https://github.com/octocat/Hello-World/pull/1347",
        "diff_url": "https://github.com/octocat/Hello-World/pull/1347.diff",
        "patch_url": "https://github.com/octocat/Hello-World/pull/1347.patch",
        "issue_url": "https://api.github.com/repos/octocat/Hello-World/issues/1347",
        "commits_url": "https://api.github.com/repos/octocat/Hello-World/pulls/1347/commits",
        "review_comments_url": "https://api.github.com/repos/octocat/Hello-World/pulls/1347/comments",
        "review_comment_url": "https://api.github.com/repos/octocat/Hello-World/pulls/comments{/number}",
        "comments_url": "https://api.github.com/repos/octocat/Hello-World/issues/1347/comments",
        "statuses_url": "https://api.github.com/repos/octocat/Hello-World/statuses/6dcb09b5b57875f334f61aebed695e2e4193db5e",
        "number": 1347,
        "state": "open",
        "locked": True,
        "title": "Amazing new feature",
        "user": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "body": "Please pull these awesome changes in!",
        "labels": [
            {
                "id": 208045946,
                "node_id": "MDU6TGFiZWwyMDgwNDU5NDY=",
                "url": "https://api.github.com/repos/octocat/Hello-World/labels/bug",
                "name": "bug",
                "description": "Something isn't working",
                "color": "f29513",
                "default": True,
            }
        ],
        "milestone": {
            "url": "https://api.github.com/repos/octocat/Hello-World/milestones/1",
            "html_url": "https://github.com/octocat/Hello-World/milestones/v1.0",
            "labels_url": "https://api.github.com/repos/octocat/Hello-World/milestones/1/labels",
            "id": 1002604,
            "node_id": "MDk6TWlsZXN0b25lMTAwMjYwNA==",
            "number": 1,
            "state": "open",
            "title": "v1.0",
            "description": "Tracking milestone for version 1.0",
            "creator": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "open_issues": 4,
            "closed_issues": 8,
            "created_at": "2011-04-10T20:09:31Z",
            "updated_at": "2014-03-03T18:58:10Z",
            "closed_at": "2013-02-12T13:22:01Z",
            "due_on": "2012-10-09T23:39:01Z",
        },
        "active_lock_reason": "too heated",
        "created_at": "2011-01-26T19:01:12Z",
        "updated_at": "2011-01-26T19:01:12Z",
        "closed_at": "2011-01-26T19:01:12Z",
        "merged_at": "2011-01-26T19:01:12Z",
        "merge_commit_sha": "e5bd3914e2e596debea16f433f57875b5b90bcd6",
        "assignee": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "assignees": [
            {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            {
                "login": "hubot",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/hubot_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/hubot",
                "html_url": "https://github.com/hubot",
                "followers_url": "https://api.github.com/users/hubot/followers",
                "following_url": "https://api.github.com/users/hubot/following{/other_user}",
                "gists_url": "https://api.github.com/users/hubot/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/hubot/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/hubot/subscriptions",
                "organizations_url": "https://api.github.com/users/hubot/orgs",
                "repos_url": "https://api.github.com/users/hubot/repos",
                "events_url": "https://api.github.com/users/hubot/events{/privacy}",
                "received_events_url": "https://api.github.com/users/hubot/received_events",
                "type": "User",
                "site_admin": True,
            },
        ],
        "requested_reviewers": [
            {
                "login": "other_user",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/other_user_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/other_user",
                "html_url": "https://github.com/other_user",
                "followers_url": "https://api.github.com/users/other_user/followers",
                "following_url": "https://api.github.com/users/other_user/following{/other_user}",
                "gists_url": "https://api.github.com/users/other_user/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/other_user/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/other_user/subscriptions",
                "organizations_url": "https://api.github.com/users/other_user/orgs",
                "repos_url": "https://api.github.com/users/other_user/repos",
                "events_url": "https://api.github.com/users/other_user/events{/privacy}",
                "received_events_url": "https://api.github.com/users/other_user/received_events",
                "type": "User",
                "site_admin": False,
            }
        ],
        "requested_teams": [
            {
                "id": 1,
                "node_id": "MDQ6VGVhbTE=",
                "url": "https://api.github.com/teams/1",
                "html_url": "https://api.github.com/teams/justice-league",
                "name": "Justice League",
                "slug": "justice-league",
                "description": "A great team.",
                "privacy": "closed",
                "permission": "admin",
                "members_url": "https://api.github.com/teams/1/members{/member}",
                "repositories_url": "https://api.github.com/teams/1/repos",
                "parent": None,
            }
        ],
        "head": {
            "label": "octocat:new-topic",
            "ref": "new-topic",
            "sha": "6dcb09b5b57875f334f61aebed695e2e4193db5e",
            "user": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "repo": {
                "id": 1296269,
                "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
                "name": "Hello-World",
                "full_name": "octocat/Hello-World",
                "owner": {
                    "login": "octocat",
                    "id": 1,
                    "node_id": "MDQ6VXNlcjE=",
                    "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                    "gravatar_id": "",
                    "url": "https://api.github.com/users/octocat",
                    "html_url": "https://github.com/octocat",
                    "followers_url": "https://api.github.com/users/octocat/followers",
                    "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                    "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                    "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                    "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                    "organizations_url": "https://api.github.com/users/octocat/orgs",
                    "repos_url": "https://api.github.com/users/octocat/repos",
                    "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                    "received_events_url": "https://api.github.com/users/octocat/received_events",
                    "type": "User",
                    "site_admin": False,
                },
                "private": False,
                "html_url": "https://github.com/octocat/Hello-World",
                "description": "This your first repo!",
                "fork": False,
                "url": "https://api.github.com/repos/octocat/Hello-World",
                "archive_url": "http://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
                "assignees_url": "http://api.github.com/repos/octocat/Hello-World/assignees{/user}",
                "blobs_url": "http://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
                "branches_url": "http://api.github.com/repos/octocat/Hello-World/branches{/branch}",
                "collaborators_url": "http://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
                "comments_url": "http://api.github.com/repos/octocat/Hello-World/comments{/number}",
                "commits_url": "http://api.github.com/repos/octocat/Hello-World/commits{/sha}",
                "compare_url": "http://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
                "contents_url": "http://api.github.com/repos/octocat/Hello-World/contents/{+path}",
                "contributors_url": "http://api.github.com/repos/octocat/Hello-World/contributors",
                "deployments_url": "http://api.github.com/repos/octocat/Hello-World/deployments",
                "downloads_url": "http://api.github.com/repos/octocat/Hello-World/downloads",
                "events_url": "http://api.github.com/repos/octocat/Hello-World/events",
                "forks_url": "http://api.github.com/repos/octocat/Hello-World/forks",
                "git_commits_url": "http://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
                "git_refs_url": "http://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
                "git_tags_url": "http://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
                "git_url": "git:github.com/octocat/Hello-World.git",
                "issue_comment_url": "http://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
                "issue_events_url": "http://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
                "issues_url": "http://api.github.com/repos/octocat/Hello-World/issues{/number}",
                "keys_url": "http://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
                "labels_url": "http://api.github.com/repos/octocat/Hello-World/labels{/name}",
                "languages_url": "http://api.github.com/repos/octocat/Hello-World/languages",
                "merges_url": "http://api.github.com/repos/octocat/Hello-World/merges",
                "milestones_url": "http://api.github.com/repos/octocat/Hello-World/milestones{/number}",
                "notifications_url": "http://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
                "pulls_url": "http://api.github.com/repos/octocat/Hello-World/pulls{/number}",
                "releases_url": "http://api.github.com/repos/octocat/Hello-World/releases{/id}",
                "ssh_url": "git@github.com:octocat/Hello-World.git",
                "stargazers_url": "http://api.github.com/repos/octocat/Hello-World/stargazers",
                "statuses_url": "http://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
                "subscribers_url": "http://api.github.com/repos/octocat/Hello-World/subscribers",
                "subscription_url": "http://api.github.com/repos/octocat/Hello-World/subscription",
                "tags_url": "http://api.github.com/repos/octocat/Hello-World/tags",
                "teams_url": "http://api.github.com/repos/octocat/Hello-World/teams",
                "trees_url": "http://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
                "clone_url": "https://github.com/octocat/Hello-World.git",
                "mirror_url": "git:git.example.com/octocat/Hello-World",
                "hooks_url": "http://api.github.com/repos/octocat/Hello-World/hooks",
                "svn_url": "https://svn.github.com/octocat/Hello-World",
                "homepage": "https://github.com",
                "language": None,
                "forks_count": 9,
                "stargazers_count": 80,
                "watchers_count": 80,
                "size": 108,
                "default_branch": "main",
                "open_issues_count": 0,
                "is_template": True,
                "topics": ["octocat", "atom", "electron", "api"],
                "has_issues": True,
                "has_projects": True,
                "has_wiki": True,
                "has_pages": False,
                "has_downloads": True,
                "archived": False,
                "disabled": False,
                "visibility": "public",
                "pushed_at": "2011-01-26T19:06:43Z",
                "created_at": "2011-01-26T19:01:12Z",
                "updated_at": "2011-01-26T19:14:43Z",
                "permissions": {"admin": False, "push": False, "pull": True},
                "allow_rebase_merge": True,
                "template_repository": None,
                "temp_clone_token": "ABTLWHOULUVAXGTRYU7OC2876QJ2O",
                "allow_squash_merge": True,
                "delete_branch_on_merge": True,
                "allow_merge_commit": True,
                "subscribers_count": 42,
                "network_count": 0,
            },
        },
        "base": {
            "label": "octocat:main",
            "ref": "main",
            "sha": "6dcb09b5b57875f334f61aebed695e2e4193db5e",
            "user": {
                "login": "octocat",
                "id": 1,
                "node_id": "MDQ6VXNlcjE=",
                "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                "gravatar_id": "",
                "url": "https://api.github.com/users/octocat",
                "html_url": "https://github.com/octocat",
                "followers_url": "https://api.github.com/users/octocat/followers",
                "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                "organizations_url": "https://api.github.com/users/octocat/orgs",
                "repos_url": "https://api.github.com/users/octocat/repos",
                "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                "received_events_url": "https://api.github.com/users/octocat/received_events",
                "type": "User",
                "site_admin": False,
            },
            "repo": {
                "id": 1296269,
                "node_id": "MDEwOlJlcG9zaXRvcnkxMjk2MjY5",
                "name": "Hello-World",
                "full_name": "octocat/Hello-World",
                "owner": {
                    "login": "octocat",
                    "id": 1,
                    "node_id": "MDQ6VXNlcjE=",
                    "avatar_url": "https://github.com/images/error/octocat_happy.gif",
                    "gravatar_id": "",
                    "url": "https://api.github.com/users/octocat",
                    "html_url": "https://github.com/octocat",
                    "followers_url": "https://api.github.com/users/octocat/followers",
                    "following_url": "https://api.github.com/users/octocat/following{/other_user}",
                    "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
                    "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
                    "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
                    "organizations_url": "https://api.github.com/users/octocat/orgs",
                    "repos_url": "https://api.github.com/users/octocat/repos",
                    "events_url": "https://api.github.com/users/octocat/events{/privacy}",
                    "received_events_url": "https://api.github.com/users/octocat/received_events",
                    "type": "User",
                    "site_admin": False,
                },
                "private": False,
                "html_url": "https://github.com/octocat/Hello-World",
                "description": "This your first repo!",
                "fork": False,
                "url": "https://api.github.com/repos/octocat/Hello-World",
                "archive_url": "http://api.github.com/repos/octocat/Hello-World/{archive_format}{/ref}",
                "assignees_url": "http://api.github.com/repos/octocat/Hello-World/assignees{/user}",
                "blobs_url": "http://api.github.com/repos/octocat/Hello-World/git/blobs{/sha}",
                "branches_url": "http://api.github.com/repos/octocat/Hello-World/branches{/branch}",
                "collaborators_url": "http://api.github.com/repos/octocat/Hello-World/collaborators{/collaborator}",
                "comments_url": "http://api.github.com/repos/octocat/Hello-World/comments{/number}",
                "commits_url": "http://api.github.com/repos/octocat/Hello-World/commits{/sha}",
                "compare_url": "http://api.github.com/repos/octocat/Hello-World/compare/{base}...{head}",
                "contents_url": "http://api.github.com/repos/octocat/Hello-World/contents/{+path}",
                "contributors_url": "http://api.github.com/repos/octocat/Hello-World/contributors",
                "deployments_url": "http://api.github.com/repos/octocat/Hello-World/deployments",
                "downloads_url": "http://api.github.com/repos/octocat/Hello-World/downloads",
                "events_url": "http://api.github.com/repos/octocat/Hello-World/events",
                "forks_url": "http://api.github.com/repos/octocat/Hello-World/forks",
                "git_commits_url": "http://api.github.com/repos/octocat/Hello-World/git/commits{/sha}",
                "git_refs_url": "http://api.github.com/repos/octocat/Hello-World/git/refs{/sha}",
                "git_tags_url": "http://api.github.com/repos/octocat/Hello-World/git/tags{/sha}",
                "git_url": "git:github.com/octocat/Hello-World.git",
                "issue_comment_url": "http://api.github.com/repos/octocat/Hello-World/issues/comments{/number}",
                "issue_events_url": "http://api.github.com/repos/octocat/Hello-World/issues/events{/number}",
                "issues_url": "http://api.github.com/repos/octocat/Hello-World/issues{/number}",
                "keys_url": "http://api.github.com/repos/octocat/Hello-World/keys{/key_id}",
                "labels_url": "http://api.github.com/repos/octocat/Hello-World/labels{/name}",
                "languages_url": "http://api.github.com/repos/octocat/Hello-World/languages",
                "merges_url": "http://api.github.com/repos/octocat/Hello-World/merges",
                "milestones_url": "http://api.github.com/repos/octocat/Hello-World/milestones{/number}",
                "notifications_url": "http://api.github.com/repos/octocat/Hello-World/notifications{?since,all,participating}",
                "pulls_url": "http://api.github.com/repos/octocat/Hello-World/pulls{/number}",
                "releases_url": "http://api.github.com/repos/octocat/Hello-World/releases{/id}",
                "ssh_url": "git@github.com:octocat/Hello-World.git",
                "stargazers_url": "http://api.github.com/repos/octocat/Hello-World/stargazers",
                "statuses_url": "http://api.github.com/repos/octocat/Hello-World/statuses/{sha}",
                "subscribers_url": "http://api.github.com/repos/octocat/Hello-World/subscribers",
                "subscription_url": "http://api.github.com/repos/octocat/Hello-World/subscription",
                "tags_url": "http://api.github.com/repos/octocat/Hello-World/tags",
                "teams_url": "http://api.github.com/repos/octocat/Hello-World/teams",
                "trees_url": "http://api.github.com/repos/octocat/Hello-World/git/trees{/sha}",
                "clone_url": "https://github.com/octocat/Hello-World.git",
                "mirror_url": "git:git.example.com/octocat/Hello-World",
                "hooks_url": "http://api.github.com/repos/octocat/Hello-World/hooks",
                "svn_url": "https://svn.github.com/octocat/Hello-World",
                "homepage": "https://github.com",
                "language": None,
                "forks_count": 9,
                "stargazers_count": 80,
                "watchers_count": 80,
                "size": 108,
                "default_branch": "main",
                "open_issues_count": 0,
                "is_template": True,
                "topics": ["octocat", "atom", "electron", "api"],
                "has_issues": True,
                "has_projects": True,
                "has_wiki": True,
                "has_pages": False,
                "has_downloads": True,
                "archived": False,
                "disabled": False,
                "visibility": "public",
                "pushed_at": "2011-01-26T19:06:43Z",
                "created_at": "2011-01-26T19:01:12Z",
                "updated_at": "2011-01-26T19:14:43Z",
                "permissions": {"admin": False, "push": False, "pull": True},
                "allow_rebase_merge": True,
                "template_repository": None,
                "temp_clone_token": "ABTLWHOULUVAXGTRYU7OC2876QJ2O",
                "allow_squash_merge": True,
                "delete_branch_on_merge": True,
                "allow_merge_commit": True,
                "subscribers_count": 42,
                "network_count": 0,
            },
        },
        "_links": {
            "self": {"href": "https://api.github.com/repos/octocat/Hello-World/pulls/1347"},
            "html": {"href": "https://github.com/octocat/Hello-World/pull/1347"},
            "issue": {"href": "https://api.github.com/repos/octocat/Hello-World/issues/1347"},
            "comments": {
                "href": "https://api.github.com/repos/octocat/Hello-World/issues/1347/comments"
            },
            "review_comments": {
                "href": "https://api.github.com/repos/octocat/Hello-World/pulls/1347/comments"
            },
            "review_comment": {
                "href": "https://api.github.com/repos/octocat/Hello-World/pulls/comments{/number}"
            },
            "commits": {
                "href": "https://api.github.com/repos/octocat/Hello-World/pulls/1347/commits"
            },
            "statuses": {
                "href": "https://api.github.com/repos/octocat/Hello-World/statuses/6dcb09b5b57875f334f61aebed695e2e4193db5e"
            },
        },
        "author_association": "OWNER",
        "draft": False,
        "merged": False,
        "mergeable": True,
        "rebaseable": True,
        "mergeable_state": "clean",
        "merged_by": {
            "login": "octocat",
            "id": 1,
            "node_id": "MDQ6VXNlcjE=",
            "avatar_url": "https://github.com/images/error/octocat_happy.gif",
            "gravatar_id": "",
            "url": "https://api.github.com/users/octocat",
            "html_url": "https://github.com/octocat",
            "followers_url": "https://api.github.com/users/octocat/followers",
            "following_url": "https://api.github.com/users/octocat/following{/other_user}",
            "gists_url": "https://api.github.com/users/octocat/gists{/gist_id}",
            "starred_url": "https://api.github.com/users/octocat/starred{/owner}{/repo}",
            "subscriptions_url": "https://api.github.com/users/octocat/subscriptions",
            "organizations_url": "https://api.github.com/users/octocat/orgs",
            "repos_url": "https://api.github.com/users/octocat/repos",
            "events_url": "https://api.github.com/users/octocat/events{/privacy}",
            "received_events_url": "https://api.github.com/users/octocat/received_events",
            "type": "User",
            "site_admin": False,
        },
        "comments": 10,
        "review_comments": 0,
        "maintainer_can_modify": True,
        "commits": 3,
        "additions": 100,
        "deletions": 3,
        "changed_files": 5,
    }
